@@ -144,11 +144,12 @@ main(int argc, char **argv)
 #endif
 
 	crm_system_name = basename(argv[0]);
-	crm_log_init(crm_system_name, FALSE);
 
 	if(argc < 2) {
 		usage(crm_system_name, LSB_EXIT_EINVAL);
 	}
+
+	crm_log_init(crm_system_name, LOG_ERR, FALSE, TRUE, argc, argv);
 	
 	while (1) {
 #ifdef HAVE_GETOPT_H
@@ -278,7 +279,6 @@ main(int argc, char **argv)
 	hb_cluster = do_init();
 	if (hb_cluster != NULL) {
 		int res = 0;
-		cl_log_args(argc, argv);
 		res = do_work(hb_cluster);
 		if (res > 0) {
 			/* wait for the reply by creating a mainloop and running it until
@@ -456,27 +456,19 @@ do_work(ll_cluster_t * hb_cluster)
 void
 crmd_ipc_connection_destroy(gpointer user_data)
 {
-	crm_info("Connection to CRMd was terminated");
-	exit(1);
+	crm_debug("Connection to CRMd was terminated");
 }
 
 
 ll_cluster_t *
 do_init(void)
 {
-	int facility;
 	GCHSource *src = NULL;
 	ll_cluster_t *hb_cluster = NULL;
 	
 	/* change the logging facility to the one used by heartbeat daemon */
 	hb_cluster = ll_cluster_new("heartbeat");
 	
-	crm_debug_2("Switching to Heartbeat logger");
-	if (( facility =
-	      hb_cluster->llc_ops->get_logfacility(hb_cluster)) > 0) {
-		cl_log_set_facility(facility);
-	}
-
 	crm_malloc0(admin_uuid, 11);
 	if(admin_uuid != NULL) {
 		snprintf(admin_uuid, 10, "%d", getpid());
@@ -503,6 +495,7 @@ do_init(void)
 gboolean
 admin_msg_callback(IPC_Channel * server, void *private_data)
 {
+	int rc = 0;
 	int lpc = 0;
 	IPC_Message *msg = NULL;
 	ha_msg_input_t *new_input = NULL;
@@ -520,9 +513,10 @@ admin_msg_callback(IPC_Channel * server, void *private_data)
 			delete_ha_msg_input(new_input);
 			new_input = NULL;
 		}
-		
-		if (server->ops->recv(server, &msg) != IPC_OK) {
-			perror("Receive failure:");
+
+		rc = server->ops->recv(server, &msg);
+		if (rc != IPC_OK) {
+		    cl_perror("Receive failure (%d)", rc);
 			return !hack_return_good;
 		}
 
@@ -632,6 +626,7 @@ admin_message_timeout(gpointer data)
 		(int)message_timeout_ms/1000);
 	crm_err("No messages received in %d seconds",
 		(int)message_timeout_ms/1000);
+	operation_status = -3;
 	g_main_quit(mainloop);
 	return FALSE;
 }
