@@ -302,10 +302,6 @@ crmd_ha_status_callback(
 		update = create_node_state(
 			node, status, XML_BOOLEAN_NO, OFFLINESTATUS,
 			CRMD_STATE_INACTIVE, NULL, TRUE, __FUNCTION__);
-		if(update) {
-			crm_xml_add(update, XML_CIB_ATTR_REPLACE,
-				    XML_TAG_TRANSIENT_NODEATTRS);
-		}
 		
 	} else if(safe_str_eq(status, ACTIVESTATUS)) {
 		update = create_node_state(
@@ -373,13 +369,11 @@ crmd_client_status_callback(const char * node, const char * client,
 		
 	} else {
 		crm_debug_3("Got client status callback");
-		update = create_node_state(
-			node, NULL, NULL, status, join,
-			NULL, clear_shutdown, __FUNCTION__);
-
-		if(clear_shutdown) {
-			crm_xml_add(update, XML_CIB_ATTR_REPLACE,
-				    XML_TAG_TRANSIENT_NODEATTRS);
+		update = create_node_state(node, NULL, NULL, status, join,
+					   NULL, clear_shutdown, __FUNCTION__);
+	
+		if(safe_str_eq(status, ONLINESTATUS)){
+		    crm_xml_add(update, XML_CIB_ATTR_REPLACE, XML_CIB_TAG_LRM",,"XML_TAG_TRANSIENT_NODEATTRS",");
 		}
 		
 		/* it is safe to keep these updates on the local node
@@ -455,8 +449,8 @@ crmd_client_connect(IPC_Channel *client_channel, gpointer user_data)
 
 		crm_debug_2("Created client: %p", blank_client);
 		
-		client_channel->ops->set_recv_qlen(client_channel, 100);
-		client_channel->ops->set_send_qlen(client_channel, 100);
+		client_channel->ops->set_recv_qlen(client_channel, 1024);
+		client_channel->ops->set_send_qlen(client_channel, 1024);
 	
 		blank_client->client_channel = client_channel;
 		blank_client->sub_sys   = NULL;
@@ -552,6 +546,7 @@ crmd_ccm_msg_callback(
 #endif
 			break;
 		case OC_EV_MS_PRIMARY_RESTORED:
+			update_cache = TRUE;
 			current_ccm_membership_id = instance;
 			if(AM_I_DC && need_transition(fsa_state)) {
 				trigger_transition = TRUE;
