@@ -32,6 +32,7 @@
 #include <fsa_proto.h>
 #include <crmd_messages.h>
 #include <crmd_callbacks.h>
+#include <crmd_lrm.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -40,7 +41,6 @@
 char *ipc_server = NULL;
 
 extern void crmd_ha_connection_destroy(gpointer user_data);
-extern gboolean verify_stopped(gboolean force, int log_level);
 
 gboolean crm_shutdown(int nsig, gpointer unused);
 gboolean register_with_ha(ll_cluster_t *hb_cluster, const char *client_name);
@@ -51,7 +51,7 @@ GHashTable   *ipc_clients = NULL;
 GTRIGSource  *fsa_source = NULL;
 
 /*	 A_HA_CONNECT	*/
-enum crmd_fsa_input
+void
 do_ha_control(long long action,
 	       enum crmd_fsa_cause cause,
 	       enum crmd_fsa_state cur_state,
@@ -82,7 +82,7 @@ do_ha_control(long long action,
 		
 		if(registered == FALSE) {
 			register_fsa_error(C_FSA_INTERNAL, I_FAIL, NULL);
-			return I_NULL;
+			return;
 		}
 		clear_bit_inplace(fsa_input_register, R_HA_DISCONNECTED);
 		crm_info("Connected to Heartbeat");
@@ -92,13 +92,10 @@ do_ha_control(long long action,
 		crm_err("Unexpected action %s in %s",
 		       fsa_action2string(action), __FUNCTION__);
 	}
-	
-	
-	return I_NULL;
 }
 
 /*	 A_SHUTDOWN	*/
-enum crmd_fsa_input
+void
 do_shutdown(long long action,
 	    enum crmd_fsa_cause cause,
 	    enum crmd_fsa_state cur_state,
@@ -128,20 +125,17 @@ do_shutdown(long long action,
 			continue_shutdown = FALSE;
 		}
 	}
-
+    
 	if(continue_shutdown == FALSE) {
 		crm_info("Waiting for subsystems to exit");
 		crmd_fsa_stall(NULL);
-
-	} else {
-		register_fsa_input(C_FSA_INTERNAL, I_TERMINATE, NULL);
 	}
 	
-	return I_NULL;
+	crm_info("All subsystems stopped, continuing");
 }
 
 /*	 A_SHUTDOWN_REQ	*/
-enum crmd_fsa_input
+void
 do_shutdown_req(long long action,
 	    enum crmd_fsa_cause cause,
 	    enum crmd_fsa_state cur_state,
@@ -163,8 +157,6 @@ do_shutdown_req(long long action,
 			register_fsa_error(C_FSA_INTERNAL, I_ERROR, NULL);
 		}
 	}
-
-	return I_NULL;
 }
 
 extern char *max_generation_from;
@@ -278,7 +270,7 @@ static void free_mem(fsa_data_t *msg_data)
 }
 
 /*	 A_EXIT_0, A_EXIT_1	*/
-enum crmd_fsa_input
+void
 do_exit(long long action,
 	enum crmd_fsa_cause cause,
 	enum crmd_fsa_state cur_state,
@@ -293,9 +285,9 @@ do_exit(long long action,
 		exit_code = 1;
 		log_level = LOG_ERR;
 		exit_type = "forcefully";
-		verify_stopped(TRUE, LOG_ERR);
 	}
 	
+	verify_stopped(cur_state, LOG_ERR);
 	do_crm_log(log_level, "Performing %s - %s exiting the CRMd",
 		      fsa_action2string(action), exit_type);
 	
@@ -313,12 +305,10 @@ do_exit(long long action,
 	crm_info("[%s] stopped (%d)", crm_system_name, exit_code);
 	cl_flush_logs();
 	exit(exit_code);
-
-	return I_NULL;
 }
 
 /*	 A_STARTUP	*/
-enum crmd_fsa_input
+void
 do_startup(long long action,
 	   enum crmd_fsa_cause cause,
 	   enum crmd_fsa_state cur_state,
@@ -426,7 +416,7 @@ do_startup(long long action,
 	
 	if(wait_timer != NULL) {
 		wait_timer->source_id = 0;
-		wait_timer->period_ms = 500;
+		wait_timer->period_ms = 2000;
 		wait_timer->fsa_input = I_NULL;
 		wait_timer->callback = crm_timer_popped;
 		wait_timer->repeat = FALSE;
@@ -508,28 +498,22 @@ do_startup(long long action,
 		g_str_hash, g_str_equal,
 		g_hash_destroy_str, g_hash_destroy_str);
 
-	set_sigchld_proctrack(G_PRIORITY_HIGH);
-	
-	return I_NULL;
+	set_sigchld_proctrack(G_PRIORITY_HIGH,DEFAULT_MAXDISPATCHTIME);
 }
 
 /*	 A_STOP	*/
-enum crmd_fsa_input
+void
 do_stop(long long action,
 	enum crmd_fsa_cause cause,
 	enum crmd_fsa_state cur_state,
 	enum crmd_fsa_input current_input,
 	fsa_data_t *msg_data)
 {
-	if(verify_stopped(FALSE, LOG_DEBUG) == FALSE) {
-		crmd_fsa_stall(NULL);
-	}
-
-	return I_NULL;
+    register_fsa_input(C_FSA_INTERNAL, I_TERMINATE, NULL);
 }
 
 /*	 A_STARTED	*/
-enum crmd_fsa_input
+void
 do_started(long long action,
 	   enum crmd_fsa_cause cause,
 	   enum crmd_fsa_state cur_state,
@@ -541,28 +525,28 @@ do_started(long long action,
 			 R_CCM_DATA);
 
 		crmd_fsa_stall(NULL);
-		return I_NULL;
+		return;
 
 	} else if(is_set(fsa_input_register, R_LRM_CONNECTED) == FALSE) {
 		crm_info("Delaying start, LRM (%.16llx) not connected",
 			 R_LRM_CONNECTED);
 
 		crmd_fsa_stall(NULL);
-		return I_NULL;
+		return;
 
 	} else if(is_set(fsa_input_register, R_CIB_CONNECTED) == FALSE) {
 		crm_info("Delaying start, CIB (%.16llx) not connected",
 			 R_CIB_CONNECTED);
 
 		crmd_fsa_stall(NULL);
-		return I_NULL;
+		return;
 
 	} else if(is_set(fsa_input_register, R_READ_CONFIG) == FALSE) {
 		crm_info("Delaying start, Config not read (%.16llx)",
 			 R_READ_CONFIG);
 
 		crmd_fsa_stall(NULL);
-		return I_NULL;
+		return;
 
 	} else if(is_set(fsa_input_register, R_PEER_DATA) == FALSE) {
 		HA_Message *	msg = NULL;
@@ -580,18 +564,16 @@ do_started(long long action,
 		
 		crm_timer_start(wait_timer);
 		crmd_fsa_stall(NULL);
-		return I_NULL;
+		return;
 	}
 
 	crm_info("The local CRM is operational");
 	clear_bit_inplace(fsa_input_register, R_STARTING);
 	register_fsa_input(msg_data->fsa_cause, I_PENDING, NULL);
-	
-	return I_NULL;
 }
 
 /*	 A_RECOVER	*/
-enum crmd_fsa_input
+void
 do_recover(long long action,
 	   enum crmd_fsa_cause cause,
 	   enum crmd_fsa_state cur_state,
@@ -602,9 +584,7 @@ do_recover(long long action,
 	crm_err("Action %s (%.16llx) not supported",
 	       fsa_action2string(action), action);
 
-	register_fsa_input(C_FSA_INTERNAL, I_STOP, NULL);
-
-	return I_NULL;
+	register_fsa_input(C_FSA_INTERNAL, I_TERMINATE, NULL);
 }
 
 pe_cluster_option crmd_opts[] = {
@@ -614,7 +594,7 @@ pe_cluster_option crmd_opts[] = {
 	{ XML_CONFIG_ATTR_ELECTION_FAIL, NULL, "time", NULL, "2min", &check_timer, "*** Advanced Use Only ***.", "If need to adjust this value, it probably indicates the presence of a bug." },
 	{ XML_CONFIG_ATTR_FORCE_QUIT, NULL, "time", NULL, "20min", &check_timer, "*** Advanced Use Only ***.", "If need to adjust this value, it probably indicates the presence of a bug." },
 	{ "crmd-integration-timeout", NULL, "time", NULL, "3min", &check_timer, "*** Advanced Use Only ***.", "If need to adjust this value, it probably indicates the presence of a bug." },
-	{ "crmd-finalization-timeout", NULL, "time", NULL, "10min", &check_timer, "*** Advanced Use Only ***.", "If need to adjust this value, it probably indicates the presence of a bug." },
+	{ "crmd-finalization-timeout", NULL, "time", NULL, "30min", &check_timer, "*** Advanced Use Only ***.", "If you need to adjust this value, it probably indicates the presence of a bug." },
 };
 
 void
@@ -713,7 +693,7 @@ config_query_callback(const HA_Message *msg, int call_id, int rc,
 }
 
 /*	 A_READCONFIG	*/
-enum crmd_fsa_input
+void
 do_read_config(long long action,
 	       enum crmd_fsa_cause cause,
 	       enum crmd_fsa_state cur_state,
@@ -725,8 +705,6 @@ do_read_config(long long action,
 
 	add_cib_op_callback(call_id, FALSE, NULL, config_query_callback);
 	crm_debug_2("Querying the CIB... call %d", call_id);
-	
-	return I_NULL;
 }
 
 
