@@ -75,8 +75,8 @@ class LinuxHAv2(HeartbeatCM):
             "QuorumCmd"      : "/usr/sbin/ccm_tool -q",
             "CibQuery"       : "/usr/sbin/cibadmin -Ql",
             "ParitionCmd"    : "/usr/sbin/ccm_tool -p",
-            "IsRscRunning"   : "/usr/lib64/heartbeat/lrmadmin -E %s monitor 0 0 EVERYTIME 2>/dev/null|grep return",
-            "ExecuteRscOp"   : "/usr/lib64/heartbeat/lrmadmin -n %s -E %s %s 0 %d EVERYTIME 2>/dev/null",
+            "IsRscRunning"   : "/usr/lib/heartbeat/lrmadmin -E %s monitor 0 0 EVERYTIME 2>/dev/null|grep return",
+            "ExecuteRscOp"   : "/usr/lib/heartbeat/lrmadmin -n %s -E %s %s 0 %d EVERYTIME 2>/dev/null",
             "CIBfile"        : "%s:/var/lib/heartbeat/crm/cib.xml",
             "TmpDir"         : "/tmp",
             "BreakCommCmd2"  : "/usr/share/heartbeat/TestHeartbeatComm break-communication %s>/dev/null 2>&1",
@@ -125,10 +125,9 @@ class LinuxHAv2(HeartbeatCM):
                 r"Search terminated:",
                 r"No need to invoke the TE",
                 r":global_timer_callback",
-                r"Updating failcount for ",
                 r"Faking parameter digest creation",
                 r"Parameters to .* action changed:",
-                r"apply_xml_diff: Diff application failed!",
+                r"Parameters to .* changed",
             ),
         })
         del self["Standby"]
@@ -163,6 +162,8 @@ class LinuxHAv2(HeartbeatCM):
             self.CIBsync[node] = 1
             self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml")
             self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml.sig")
+            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml.last")
+            self.rsh.remote_py(node, "os", "system", "rm -f /var/lib/heartbeat/crm/cib.xml.sig.last")
 
             # Only install the CIB on the first node, all the other ones will pick it up from there
             if self.cib_installed == 1:
@@ -275,10 +276,16 @@ class LinuxHAv2(HeartbeatCM):
                 self.rsh.readaline(node, (self["StatusCmd"] %node) )
                 any_up = 1
 
-        if any_up == 0 or idle_watch.look():
+        if any_up == 0:
+            self.debug("Cluster is inactive") 
             return 1
 
-        self.log("Warn: Cluster Master not IDLE") 
+        ret = idle_watch.look()
+        if ret:
+            self.debug(ret) 
+            return 1
+
+        self.log("Warn: Cluster Master not IDLE after %ds" % timeout) 
         return None
 
     def is_node_dc(self, node, status_line=None):
@@ -489,21 +496,111 @@ class LinuxHAv2(HeartbeatCM):
 
         return 0
     def Components(self):    
-        complist = [Process("lrmd",self),Process("crmd",self)]
+        complist = []
+        common_ignore = [
+                    "Pending action:",
+                    "ERROR: crm_log_message_adv:",
+                    "ERROR: MSG: No message to dump",
+                    "pending LRM operations at shutdown",
+                    "Lost connection to the CIB service",
+                    "Connection to the CIB terminated...",
+                    "Sending message to CIB service FAILED",
+                    "crmd: .*Action A_RECOVER .* not supported",
+                    "ERROR: stonithd_op_result_ready: not signed on",
+                    "send_ipc_message: IPC Channel to .* is not connected",
+                    "unconfirmed_actions: Waiting on .* unconfirmed actions",
+                    "cib_native_msgready: Message pending on command channel",
+                    "crmd:.*do_exit: Performing A_EXIT_1 - forcefully exiting the CRMd",
+                    "verify_stopped: Resource .* was active at shutdown.  You may ignore this error if it is unmanaged.",
+            ]
+
+        stonith_ignore = [
+            "ERROR: stonithd_signon: ",
+            "update_failcount: Updating failcount for child_DoFencing",
+            "ERROR: te_connect_stonith: Sign-in failed: triggered a retry",
+            ]
+
+        stonith_ignore.extend(common_ignore)
+
+        complist.append(Process("ccm", 0, [
+                    "State transition S_IDLE",
+                    "CCM connection appears to have failed",
+                    "crmd: .*Action A_RECOVER .* not supported",
+                    "crmd: .*Input I_TERMINATE from do_recover",
+                    "Exiting to recover from CCM connection failure",
+                    "crmd:.*do_exit: Could not recover from internal error",
+                    "crmd: .*I_ERROR.*(ccm_dispatch|crmd_cib_connection_destroy)",
+#                    "WARN: determine_online_status: Node .* is unclean",
+#                    "Scheduling Node .* for STONITH",
+#                    "Executing .* fencing operation",
+#                    "tengine_stonith_callback: .*result=0",
+                    "State transition S_.* -> S_INTEGRATION.*input=I_NODE_JOIN",
+                    "State transition S_STARTING -> S_PENDING",
+                    ], [], common_ignore, 1, self))
+
+        complist.append(Process("cib", 0, [
+                    "State transition S_IDLE",
+                    "Lost connection to the CIB service",
+                    "Connection to the CIB terminated...",
+                    "crmd: .*Input I_TERMINATE from do_recover",
+                    "crmd: .*I_ERROR.*crmd_cib_connection_destroy",
+                    "crmd:.*do_exit: Could not recover from internal error",
+                    ], [], common_ignore, 1, self))
+
+        complist.append(Process("lrmd", 0, [
+                    "State transition S_IDLE",
+                    "LRM Connection failed",
+                    "crmd: .*I_ERROR.*lrm_dispatch",
+                    "State transition S_STARTING -> S_PENDING",
+                    ".*crmd .*exited with return code 2.",
+                    "crmd: .*Input I_TERMINATE from do_recover",
+                    "crmd:.*do_exit: Could not recover from internal error",
+                    ], [], common_ignore, 1, self))
+
+        complist.append(Process("crmd", 0, [
+#                    "WARN: determine_online_status: Node .* is unclean",
+#                    "Scheduling Node .* for STONITH",
+#                    "Executing .* fencing operation",
+#                    "tengine_stonith_callback: .*result=0",
+                    "State transition S_IDLE",
+                    "State transition S_STARTING -> S_PENDING",
+                    ], [
+                    "tengine: .*ERROR: subsystem_msg_dispatch: The server .* has left us: Shutting down...NOW",
+                    "pengine: .*ERROR: subsystem_msg_dispatch: The server .* has left us: Shutting down...NOW",
+                    ], common_ignore, 1, self))
+
+        complist.append(Process("pengine", 1, [
+                    "State transition S_IDLE",
+                    ".*crmd .*exited with return code 2.",
+                    "crmd: .*Input I_TERMINATE from do_recover",
+                    "crmd:.*do_exit: Could not recover from internal error",
+                    ], [], common_ignore, 1, self))
+
+        complist.append(Process("tengine", 1, [
+                    "State transition S_IDLE",
+                    ".*crmd .*exited with return code 2.",
+                    "crmd: .*Input I_TERMINATE from do_recover",
+                    "crmd:.*do_exit: Could not recover from internal error",
+                    ], [], common_ignore, 1, self))
+
         if self.Env["DoFencing"] == 1 :
-            complist.append(Process("stonithd",self))
-            complist.append(Process("heartbeat",self))
+            complist.append(Process("stonithd", 0, [], [
+                        "tengine_stonith_connection_destroy: Fencing daemon has left us",
+                        "Attempting connection to fencing daemon",
+                        "te_connect_stonith: Connected",
+                        ], stonith_ignore, 0, self))
+#            complist.append(Process("heartbeat", 0, [], [], [], None, self))
         return complist
-    
+
     def NodeUUID(self, node):
         lines = self.rsh.readlines(node, self["UUIDQueryCmd"])
         for line in lines:
-            self.debug("UUIDLine:"+ line) 
+            self.debug("UUIDLine:"+ line)
             m = re.search(r'%s.+\((.+)\)' % node, line)
-            if m:   
+            if m:
                 return m.group(1)
-        return ""       
-                            
+        return ""
+
     def StandbyStatus(self, node):
         out=self.rsh.readaline(node, self["StandbyQueryCmd"]%node)
         if not out:
