@@ -824,7 +824,7 @@ class ClusterManager(UserDict):
     def TestLogging(self):
         self.TestLoggingLevel=self.TestLoggingLevel+1
         ret=1
-        if self.TestLoggingLevel > 2:
+        if self.TestLoggingLevel > 3:
             self.log("ERROR: Unable to fix remote logging. Stopping tests.")
             self.TestLoggingLevel=self.TestLoggingLevel-1
             return None
@@ -846,7 +846,7 @@ class ClusterManager(UserDict):
             for regex in watch.unmatched:
                 self.log ("ERROR: Test message [%s] not found in logs." % (regex))
             self.RestartClusterLogging()
-            time.sleep(30)
+            time.sleep(30*self.TestLoggingLevel)
             ret=self.TestLogging()
             if ret:
               self.log("NOTE: Cluster logging now working.")
@@ -939,14 +939,19 @@ class Component:
         None
         
 class Process(Component):
-    def __init__(self, name, cm):
+    def __init__(self, name, dc_only, pats, dc_pats, badnews_ignore, triggersreboot, cm):
         self.name = str(name)
+        self.dc_only = dc_only
+        self.pats = pats
+        self.dc_pats = dc_pats
         self.CM = cm
+        self.badnews_ignore = badnews_ignore
+	self.triggersreboot = triggersreboot
         self.KillCmd = "killall -9 " + self.name
         
     def kill(self, node):
         if self.CM.rsh(node, self.KillCmd) != 0:
-            self.log ("ERROR: Kill %s failed on node %s" %(name,node))
+            self.CM.log ("ERROR: Kill %s failed on node %s" %(name,node))
             return None
         return 1
 
