@@ -35,6 +35,7 @@ from socket import gethostbyname_ex
 tests = None
 cm = None
 old_handler = None
+DefaultFacility = "daemon"
 
 def sig_handler(signum, frame) :
     if cm != None:
@@ -70,15 +71,15 @@ class Stonithd(ResetMechanism):
         self.sttype = sttype
         self.nodes = nodes
 
-        self.query_cmd_pat   = '/usr/lib64/heartbeat/stonithdtest/apitest 0 %s 20000 0'
-        self.reset_cmd_pat   = '/usr/lib64/heartbeat/stonithdtest/apitest 1 %s 20000 0'
-        self.poweron_cmd_pat = '/usr/lib64/heartbeat/stonithdtest/apitest 2 %s 20000 0'
-        self.poweroff_cmd_pat= '/usr/lib64/heartbeat/stonithdtest/apitest 3 %s 20000 0'
+        self.query_cmd_pat   = '/usr/lib/heartbeat/stonithdtest/apitest 0 %s 60000 0'
+        self.reset_cmd_pat   = '/usr/lib/heartbeat/stonithdtest/apitest 1 %s 60000 0'
+        self.poweron_cmd_pat = '/usr/lib/heartbeat/stonithdtest/apitest 2 %s 60000 0'
+        self.poweroff_cmd_pat= '/usr/lib/heartbeat/stonithdtest/apitest 3 %s 60000 0'
 
-        self.lrmd_add_pat    = '/usr/lib64/heartbeat/lrmadmin -A %s stonith ' + sttype + ' NULL hostlist=%s'
-        self.lrmd_start_pat  = '/usr/lib64/heartbeat/lrmadmin -E %s start 0 0 EVERYTIME'
-        self.lrmd_stop_pat   = '/usr/lib64/heartbeat/lrmadmin -E %s stop 0 0 EVERYTIME'
-        self.lrmd_del_pat    = '/usr/lib64/heartbeat/lrmadmin -D %s'
+        self.lrmd_add_pat    = '/usr/lib/heartbeat/lrmadmin -A %s stonith ' + sttype + ' NULL hostlist=%s livedangerously=yes'
+        self.lrmd_start_pat  = '/usr/lib/heartbeat/lrmadmin -E %s start 0 0 EVERYTIME'
+        self.lrmd_stop_pat   = '/usr/lib/heartbeat/lrmadmin -E %s stop 0 0 EVERYTIME'
+        self.lrmd_del_pat    = '/usr/lib/heartbeat/lrmadmin -D %s'
 
         self.rsc_id          = 'my_stonithd_id'
 
@@ -147,7 +148,7 @@ class Stonithd(ResetMechanism):
         elif action == "POWERON":
             command = self.poweron_cmd_pat % target_node
         else:
-            self.fail_reason = "unknown opration type %s" % action
+            self.fail_reason = "unknown operation type %s" % action
             return 0
 
         lines = self._remote_readlines(init_node, command)
@@ -192,12 +193,11 @@ class Logger:
 class SysLog(Logger):
     # http://docs.python.org/lib/module-syslog.html
     defaultsource="CTS"
-    defaultfacility= syslog.LOG_LOCAL7
     map = {
             "kernel":   syslog.LOG_KERN,
             "user":     syslog.LOG_USER,
             "mail":     syslog.LOG_MAIL,
-            "daemon":   syslog.LOG_MAIL,
+            "daemon":   syslog.LOG_DAEMON,
             "auth":     syslog.LOG_AUTH,
             "lpr":      syslog.LOG_LPR,
             "news":     syslog.LOG_NEWS,
@@ -221,10 +221,11 @@ class SysLog(Logger):
 
         if labinfo.has_key("SyslogFacility"):
             self.facility=labinfo["SyslogFacility"]
-            if SysLog.map.has_key(self.facility):
-              self.facility=SysLog.map[self.facility]
         else:
-            self.facility=SysLog.defaultfacility
+            self.facility=DefaultFacility
+
+        if SysLog.map.has_key(self.facility):
+            self.facility=SysLog.map[self.facility]
 
         syslog.openlog(self.source, 0, self.facility)
 
@@ -350,7 +351,7 @@ class CtsLab(UserDict):
         if not self.has_key("logrestartcmd"):
             self["logrestartcmd"] = "/etc/init.d/syslog restart"
         if not self.has_key("logfacility"):
-            LogFacility = "local7"
+            LogFacility = DefaultFacility
 
         #
         #  Now set up our random number generator...
@@ -509,7 +510,7 @@ if __name__ == '__main__':
     import CM_hb
 
     HAdir = "/etc/ha.d"
-    LogFile = "/var/log/ha-log-local7"
+    LogFile = "/var/log/ha-log-"+DefaultFacility
     DoStonith = 1
     DoStandby = 1
     DoFencing = 1
@@ -532,7 +533,7 @@ if __name__ == '__main__':
     ListTests = 0
     HaveSeed = 0
     oprofile = None
-    StonithType = "ssh"
+    StonithType = "external/ssh"
     StonithParams = None
     StonithParams = "hostlist=dynamic".split('=')
 
@@ -687,7 +688,7 @@ if __name__ == '__main__':
         if config.Parameters.has_key("logfacility"): 
             LogFacility = config.Parameters["logfacility"][0]
         else:
-            LogFacility = "local7"
+            LogFacility = DefaultFacility
 
     if LimitNodes > 0:
         if len(node_list) > LimitNodes:
@@ -774,6 +775,7 @@ if __name__ == '__main__':
     if Environment.has_key("SuppressMonitoring") \
     and Environment["SuppressMonitoring"]:
         cm.log("Resource Monitoring is disabled")
+    cm.ns.WaitForAllNodesToComeUp(config.Parameters["node"])
     cm.log("Cluster nodes: ")
     for node in config.Parameters["node"]:
         (rc, lines) = cm.rsh.remote_py(node, "os", "system", 
@@ -784,10 +786,10 @@ if __name__ == '__main__':
             out=lines[0]
             out = out[:-1]
             cm.log("    * %s: %s" % (node, out))
-        
+
     Audits = AuditList(cm)
     Tests = []
-    
+
     if Environment["DoBSC"]:
         test = BSC_AddResource(cm)
         Tests.append(test)
