@@ -334,7 +334,7 @@ typedef enum {
 	PIL_BADTYPE=2,	/* Bad plugin/interface type */
 	PIL_EXIST=3,	/* Duplicate Plugin/Interface name */
 	PIL_OOPS=4,	/* Internal Error */
-	PIL_NOPLUGIN=5,	/* No such plugin or Interface */
+	PIL_NOPLUGIN=5	/* No such plugin or Interface */
 }PIL_rc;			/* Return code from Plugin fns*/
 
 const char * PIL_strerror(PIL_rc rc);
@@ -367,7 +367,7 @@ typedef PIL_rc(*PILInterfaceFun)(PILInterface*, void* ud_interface);
 #define IS_PILINTERFACEUNIV(s)	((s)->MagicNum == PIL_MAGIC_INTERFACEUNIV)
 
 /* This is the place where PILS installs its own plugins */
-#define	PILS_BASE_PLUGINDIR	"/usr/lib64/pils/plugins"
+#define	PILS_BASE_PLUGINDIR	"/usr/lib/pils/plugins"
 
 /* The type of a Plugin Initialization Function */
 typedef PIL_rc (*PILPluginInitFun) (PILPlugin*us
@@ -396,7 +396,7 @@ typedef enum {
 	PIL_CRIT	= 2,	/* Critical -- serious error */
 	PIL_WARN	= 3,	/* Warning */
 	PIL_INFO	= 4,	/* Informative message */
-	PIL_DEBUG= 5,	/* Debug message */
+	PIL_DEBUG= 5	/* Debug message */
 }PILLogLevel;
 typedef void (*PILLogFun)(PILLogLevel priority, const char * fmt, ...);
 
@@ -443,6 +443,7 @@ struct PILPluginImports_s {
 
 	void	(*log)	(PILLogLevel priority, const char * fmt, ...);
 	void*	(*alloc)(unsigned long size);
+	void*	(*mrealloc)(void * ptr, unsigned long size);
 	void	(*mfree)(void* space);
 	char*	(*mstrdup)(const char *s);
 };
@@ -462,10 +463,25 @@ void PILCallLog(PILLogFun logfun, PILLogLevel priority, const char * fmt, ...)
 /* Create a new plugin universe - start the plugin loading system up */
 PILPluginUniv*	NewPILPluginUniv(const char * baseplugindirectory);
 
+/* Change memory allocation functions right after creating universe */
+void PilPluginUnivSetMemalloc(PILPluginUniv*
+,	void*	(*alloc)(unsigned long size)
+,	void*	(*mrealloc)(void *, unsigned long size)
+,	void	(*mfree)(void* space)
+,	char*	(*mstrdup)(const char *s));
+
+
+void PilPluginUnivSetLog(PILPluginUniv*
+,	void	(*log)	(PILLogLevel priority, const char * fmt, ...));
+
+
 /* Delete a plugin universe - shut the plugin loading system down */
 /*	Best if used carefully ;-) */
 void		DelPILPluginUniv(PILPluginUniv*);
 
+/* Set the debug level for the plugin system itself */
+void		PILpisysSetDebugLevel (int level);
+
 /* Return a list of plugins of the given type */
 char **		PILListPlugins(PILPluginUniv* u, const char *plugintype
 ,		int* plugincount /*can be NULL*/);
@@ -511,29 +527,18 @@ void PILLogMemStats(void);
 /*
  *      These functions are standard exported functions for all plugins.
  */
-#define PIL_PLUGIN_BOILERPLATE(PluginVersion, DebugName, CloseName) \
+
+#define PIL_PLUGIN_BOILERPLATE_PROTOTYPES_GENERIC(PluginVersion, DebugName) \
 /*								\
  * Prototypes for boilerplate functions				\
  */								\
 static const char*      Ourpluginversion(void);			\
 static int              GetOurDebugLevel(void);			\
 static void             SetOurDebugLevel(int);			\
-static void             CloseName(PILPlugin*);			\
 static const char *	ReturnOurLicense(void);			\
-static const char *	ReturnOurLicenseURL(void);		\
-								\
-								\
-/*								\
- * Initialize Plugin Exports structure				\
- */								\
-static PILPluginOps OurPIExports =				\
-{	Ourpluginversion					\
-,	GetOurDebugLevel					\
-,	SetOurDebugLevel					\
-,	ReturnOurLicense					\
-,	ReturnOurLicenseURL					\
-,	CloseName						\
-};								\
+static const char *	ReturnOurLicenseURL(void);
+
+#define	PIL_PLUGIN_BOILERPLATE_FUNCS(PluginVersion, DebugName)	\
 /*								\
  * Definitions of boilerplate functions				\
  */								\
@@ -559,6 +564,38 @@ static const char *						\
 ReturnOurLicenseURL(void)					\
 { return PIL_PLUGINLICENSEURL; }
 
+#define PIL_PLUGIN_BOILERPLATE(PluginVersion, DebugName, CloseName) \
+PIL_PLUGIN_BOILERPLATE_PROTOTYPES_GENERIC(PluginVersion, DebugName) \
+static void             CloseName(PILPlugin*);			\
+/*								\
+ * Initialize Plugin Exports structure				\
+ */								\
+static PILPluginOps OurPIExports =				\
+{	Ourpluginversion					\
+,	GetOurDebugLevel					\
+,	SetOurDebugLevel					\
+,	ReturnOurLicense					\
+,	ReturnOurLicenseURL					\
+,	CloseName						\
+};								\
+PIL_PLUGIN_BOILERPLATE_FUNCS(PluginVersion, DebugName)
+
+#define PIL_PLUGIN_BOILERPLATE2(PluginVersion, DebugName)	\
+PIL_PLUGIN_BOILERPLATE_PROTOTYPES_GENERIC(PluginVersion, DebugName) \
+/*								\
+ * Initialize Plugin Exports structure				\
+ */								\
+static PILPluginOps OurPIExports =				\
+{	Ourpluginversion					\
+,	GetOurDebugLevel					\
+,	SetOurDebugLevel					\
+,	ReturnOurLicense					\
+,	ReturnOurLicenseURL					\
+,	NULL							\
+};								\
+PIL_PLUGIN_BOILERPLATE_FUNCS(PluginVersion, DebugName)
+
+
 /* A few sample licenses and URLs.  We can easily add to this */
 
 #define	LICENSE_GPL	 "gpl"
