@@ -210,6 +210,7 @@ struct IPC_Stats {
 };
 
 struct IPC_Stats	SocketIPCStats = {0,0,0,0};
+extern int	debug_level;
 
 /* unix domain socket implementations of IPC functions. */
 
@@ -621,13 +622,35 @@ socket_destroy_wait_conn(struct IPC_WAIT_CONNECTION * wait_conn)
 
 	if (wc != NULL) {
 #if HB_IPC_METHOD == HB_IPC_SOCKET
-		close(wc->s);
-		cl_poll_ignore(wc->s);
-		unlink(wc->path_name);
+		if (wc->s >= 0) {
+			if (debug_level > 1) {
+				cl_log(LOG_DEBUG
+				,	"%s: closing socket %d"
+				,	__FUNCTION__, wc->s);
+			}
+			close(wc->s);
+			cl_poll_ignore(wc->s);
+			unlink(wc->path_name);
+			wc->s = -1;
+		}
 #elif HB_IPC_METHOD == HB_IPC_STREAM
-		close(wc->pipefds[0]);
-		close(wc->pipefds[1]);
 		cl_poll_ignore(wc->pipefds[0]);
+		if (wc->pipefds[0] >= 0) {
+			if (debug_level > 1) {
+				cl_log(LOG_DEBUG
+				,	"%s: closing pipe[0] %d"
+				,	__FUNCTION__, wc->pipefds[0]);
+			}
+			wc->pipefds[0] = -1;
+		}
+		if (wc->pipefds[1] >= 0) {
+			if (debug_level > 1) {
+				cl_log(LOG_DEBUG
+				,	"%s: closing pipe[1] %d"
+				,	__FUNCTION__, wc->pipefds[1]);
+			}
+			wc->pipefds[0] = -1;
+		}
 		unlink(wc->path_name);
 #endif
 		g_free(wc);
@@ -660,6 +683,7 @@ socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 	struct SOCKET_WAIT_CONN_PRIVATE*	conn_private;
 	struct SOCKET_CH_PRIVATE *		ch_private ;
 	int auth_result = IPC_FAIL;
+	int					saveerrno=errno;
 	gboolean was_error = FALSE;
 #if HB_IPC_METHOD == HB_IPC_SOCKET
 	/* make peer_addr a pointer so it can be used by the
@@ -692,9 +716,11 @@ socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 		new_sock = strrecvfd.fd;
 	}
 #endif
+	saveerrno=errno;
 	if (new_sock == -1){
 		if (errno != EAGAIN && errno != EWOULDBLOCK) {
-			cl_perror("socket_accept_connection: accept");
+			cl_perror("socket_accept_connection: accept(sock=%d)"
+			,	s);
 		}
 		was_error = TRUE;
 		
@@ -727,14 +753,15 @@ socket_accept_connection(struct IPC_WAIT_CONNECTION * wait_conn
 			ch->ch_status = IPC_CONNECT;
 			ch->farside_pid = socket_get_farside_pid(new_sock);
 			return ch;
-
 		}
+		saveerrno=errno;
 	}
   
 #if HB_IPC_METHOD == HB_IPC_SOCKET
 	g_free(peer_addr);
 	peer_addr = NULL;
 #endif
+	errno=saveerrno;
 	return NULL;
 
 }
@@ -759,6 +786,12 @@ socket_disconnect(struct IPC_CHANNEL* ch)
 	struct SOCKET_CH_PRIVATE* conn_info;
 
 	conn_info = (struct SOCKET_CH_PRIVATE*) ch->ch_private;
+	if (debug_level > 1) {
+		cl_log(LOG_DEBUG
+		,	"%s(sock=%d, ch=0x%lx){"
+		,	__FUNCTION__
+		,	conn_info->s, (unsigned long)ch);
+	}
 #if 0
 	if (ch->ch_status != IPC_DISCONNECT) {
   		cl_log(LOG_INFO, "forced disconnect for fd %d", conn_info->s);
@@ -767,10 +800,22 @@ socket_disconnect(struct IPC_CHANNEL* ch)
 	if (ch->ch_status == IPC_CONNECT) {
 		socket_resume_io(ch);		
 	}
-	close(conn_info->s);
-	cl_poll_ignore(conn_info->s);
-	conn_info->s = -1;
+	
+	if (conn_info->s >= 0) {
+		if (debug_level > 1) {
+			cl_log(LOG_DEBUG
+			,	"%s: closing socket %d"
+			,	__FUNCTION__, conn_info->s);
+		}
+		close(conn_info->s);
+		cl_poll_ignore(conn_info->s);
+		conn_info->s = -1;
+	}
 	ch->ch_status = IPC_DISCONNECT;
+	if (debug_level > 1) {
+		cl_log(LOG_DEBUG, "}/*%s(sock=%d, ch=0x%lx)*/"
+		,	__FUNCTION__, conn_info->s, (unsigned long)ch);
+	}
 	return IPC_OK;
 }
 
@@ -798,9 +843,17 @@ socket_destroy_queue(struct IPC_QUEUE * q)
 static void
 socket_destroy_channel(struct IPC_CHANNEL * ch)
 {
+	--ch->refcount;
+	if (ch->refcount > 0) {
+		return;
+	}
 	if (ch->ch_status == IPC_CONNECT){
 		socket_resume_io(ch);		
 	}
+	if (debug_level > 1) {
+		cl_log(LOG_DEBUG, "socket_destroy(ch=0x%lx){"
+		,	(unsigned long)ch);
+	}
 	socket_disconnect(ch);
 	socket_destroy_queue(ch->send_queue);
 	socket_destroy_queue(ch->recv_queue);
@@ -822,6 +875,10 @@ socket_destroy_channel(struct IPC_CHANNEL * ch)
 	}
 	memset(ch, 0xff, sizeof(*ch));
 	g_free((void*)ch);
+	if (debug_level > 1) {
+		cl_log(LOG_DEBUG, "}/*socket_destroy(ch=0x%lx)*/"
+		,	(unsigned long)ch);
+	}
 }
 
 static int
@@ -973,10 +1030,14 @@ socket_send(struct IPC_CHANNEL * ch, struct IPC_MESSAGE* msg)
 
 	if ( !ch->should_send_block &&
 	    ch->send_queue->current_qlen >= ch->send_queue->max_qlen) {
-		/*cl_log(LOG_WARNING, "send queue maximum length(%d) exceeded",
-		  ch->send_queue->max_qlen );*/
+		cl_log(LOG_WARNING, "send queue maximum length(%d) exceeded",
+		       (int)ch->send_queue->max_qlen);
 
-		return IPC_FAIL;
+		if (ch->should_block_fail) {
+			return IPC_FAIL;
+		} else {
+			return IPC_OK;
+		}
 	}
 	
 	while (ch->send_queue->current_qlen >= ch->send_queue->max_qlen){
@@ -2124,12 +2185,14 @@ channel_new(int sockfd, int conntype, const char *path_name) {
   temp_ch->msgpad = sizeof(struct SOCKET_MSG_HEAD);
   temp_ch->bytes_remaining = 0;
   temp_ch->should_send_block = FALSE;
+  temp_ch->should_block_fail = TRUE;
   temp_ch->send_queue = socket_queue_new();
   temp_ch->recv_queue = socket_queue_new();
   temp_ch->pool = NULL;
   temp_ch->high_flow_mark = temp_ch->send_queue->max_qlen;
   temp_ch->low_flow_mark = -1;
   temp_ch->conntype = conntype;
+  temp_ch->refcount = 0;
 
   return temp_ch;
   
