@@ -1,7 +1,22 @@
-/* $Id: ipctest.c,v 1.22.2.7 2005/11/22 17:27:02 alan Exp $ */
+/*
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
 #undef _GNU_SOURCE  /* in case it was defined on the command line */
 #define _GNU_SOURCE
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -10,13 +25,17 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+/* libgen.h: for 'basename()' on Solaris */
+#include <libgen.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
 #include <clplumbing/cl_poll.h>
 #include <clplumbing/GSource.h>
 #include <clplumbing/ipc.h>
+#include <clplumbing/cl_malloc.h>
 
 #define	MAXERRORS	1000
+#define	MAXERRORS_RECV	10
 
 typedef int (*TestFunc_t)(IPC_Channel*chan, int count);
 
@@ -24,6 +43,7 @@ static int channelpair(TestFunc_t client, TestFunc_t server, int count);
 #if 0
 static void clientserverpair(IPC_Channel* channels[2]);
 #endif
+
 static int echoserver(IPC_Channel*, int repcount);
 static int echoclient(IPC_Channel*, int repcount);
 static int asyn_echoserver(IPC_Channel*, int repcount);
@@ -37,6 +57,24 @@ static void checkifblocked(IPC_Channel* channel);
 static int (*PollFunc)(struct pollfd * fds, unsigned int, int)
 =	(int (*)(struct pollfd * fds, unsigned int, int))  poll;
 static gboolean checkmsg(IPC_Message* rmsg, const char * who, int rcount);
+
+const char *procname;
+
+int iter_def = 10000;	/* number of iterations */
+int verbosity = 0;	/* verbosity level */
+
+/*
+ * The ipc interface can be invoked as either:
+ * 1. pair (pipe/socketpair);
+ * 2. separate connect/accept (like server with multiple independent clients).
+ *
+ * If number of clients is given as 0, the "pair" mechanism is used,
+ * otherwise the client/server mechanism.
+ */
+/* *** CLIENTS_MAX currently 1 while coding *** */
+#define CLIENTS_MAX 1	/* max. number of independent clients */
+int clients_def = 0;	/* number of independent clients */
+
 static int
 channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 {
@@ -44,12 +82,20 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 	int		rc  = 0;
 	int		waitstat = 0;
 
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d: main process",
+		  procname, (int)getpid(), __LINE__);
+	}
 	switch (fork()) {
 		case -1:
 			cl_perror("can't fork");
 			exit(1);
 			break;
 		default: /* Parent */
+			if (verbosity >= 1) {
+				cl_log(LOG_DEBUG, "%s[%d]%d: main waiting...",
+				  procname, (int)getpid(), __LINE__);
+			}
 			while (wait(&waitstat) > 0) {
 				if (WIFEXITED(waitstat)) {
 					rc += WEXITSTATUS(waitstat);
@@ -57,6 +103,10 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 					rc += 1;
 				}
 			}
+			if (verbosity >= 1) {
+				cl_log(LOG_DEBUG, "%s[%d]%d: main ended rc: %d",
+				  procname, (int)getpid(), __LINE__, rc);
+			}
 			if (rc > 127) {
 				rc = 127;
 			}
@@ -81,7 +131,15 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 		case 0:		/* echo "client" Child */
 			channels[1]->ops->destroy(channels[1]);
 			channels[1] = NULL;
+			if (verbosity >= 1) {
+				cl_log(LOG_DEBUG, "%s[%d]%d: client starting...",
+				  procname, (int)getpid(), __LINE__);
+			}
 			rc = clientfunc(channels[0], count);
+			if (verbosity >= 1) {
+				cl_log(LOG_DEBUG, "%s[%d]%d: client ended rc:%d",
+				  procname, (int)getpid(), __LINE__, rc);
+			}
 			exit (rc > 127 ? 127 : rc);
 			break;
 
@@ -90,6 +148,10 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 	}
 	channels[0]->ops->destroy(channels[0]);
 	channels[0] = NULL;
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d: server starting...",
+		  procname, (int)getpid(), __LINE__);
+	}
 	rc = serverfunc(channels[1], count);
 	wait(&waitstat);
 	if (WIFEXITED(waitstat)) {
@@ -97,31 +159,200 @@ channelpair(TestFunc_t	clientfunc, TestFunc_t serverfunc, int count)
 	}else{
 		rc += 1;
 	}
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d: server ended rc:%d",
+		  procname, (int)getpid(), __LINE__, rc);
+	}
 	return(rc);
 }
 
-#if 0
-static void
-clientserverpair(IPC_Channel* channels[2])
+/* server with many clients */
+static int
+clientserver(TestFunc_t clientfunc, TestFunc_t serverfunc, int count, int clients)
 {
-	char			path[] = IPC_PATH_ATTR;
-	char			commpath[] = "/var/run/foobar"
-	GHashTable *		wattrs;
-	IPC_WAIT_CONNECTION*	wconn;
+	IPC_Channel* channel;
+	int rc  = 0;
+	int waitstat = 0;
+	struct IPC_WAIT_CONNECTION *wconn;
+	char path[] = IPC_PATH_ATTR;
+	char commpath[] = "/tmp/foobar";	/* *** CHECK/FIX: Is this OK? */
+	GHashTable * wattrs;
+	int i;
+	pid_t pid;
 
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d: main process",
+		  procname, (int)getpid(), __LINE__);
+	}
+
+	switch (fork()) {
+		case -1:
+			cl_perror("can't fork");
+			exit(1);
+			break;
+		default: /* Parent */
+			if (verbosity >= 1) {
+				cl_log(LOG_DEBUG, "%s[%d]%d: main waiting...",
+				  procname, (int)getpid(), __LINE__);
+			}
+			while ((pid = wait(&waitstat)) > 0) {
+				if (WIFEXITED(waitstat)) {
+					rc += WEXITSTATUS(waitstat);
+				}else{
+					rc += 1;
+				}
+			}
+			if (verbosity >= 1) {
+				cl_log(LOG_DEBUG, "%s[%d]%d: main ended rc: %d",
+				  procname, (int)getpid(), __LINE__, rc);
+			}
+			if (rc > 127) {
+				rc = 127;
+			}
+			exit(rc);
+			break;
+		case 0: /* Child */
+			break;
+	}
+
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d:",
+		  procname, (int)getpid(), __LINE__);
+	}
+
+	/* set up a server */
 	wattrs = g_hash_table_new(g_str_hash, g_str_equal);
-
+	if (! wattrs) {
+		cl_perror("g_hash_table_new() failed");
+		exit(1);
+	}
 	g_hash_table_insert(wattrs, path, commpath);
 
-	wconn = ipc_wait_conn_constructor(IPC_ANYTYPE, wconnattrs);
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d:",
+		  procname, (int)getpid(), __LINE__);
+	}
 
-	if (wconn == NULL) {
-		cl_perror("Can't create wait connection");
+	wconn = ipc_wait_conn_constructor(IPC_ANYTYPE, wattrs);
+	if (! wconn) {
+		cl_perror("could not establish server");
 		exit(1);
 	}
 
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d:",
+		  procname, (int)getpid(), __LINE__);
+	}
+
+	/* spawn the clients */
+	for (i = 1; i <= clients; i++) {
+		if (verbosity >= 1) {
+			cl_log(LOG_DEBUG, "%s[%d]%d: fork client %d of %d",
+			  procname, (int)getpid(), __LINE__, i, clients);
+		}
+		switch (fork()) {
+			case -1:
+				cl_perror("can't fork");
+				exit(1);
+				break;
+
+			case 0:	/* echo "client" Child */
+				if (verbosity >= 1) {
+					cl_log(LOG_DEBUG, "%s[%d]%d: client %d starting...",
+					  procname, (int)getpid(), __LINE__, i);
+				}
+				channel = ipc_channel_constructor(IPC_ANYTYPE, wattrs);
+				if (channel == NULL) {
+					cl_perror("client: channel creation failed");
+					exit(1);
+				}
+
+				rc = channel->ops->initiate_connection(channel);
+				if (rc != IPC_OK) {
+					cl_perror("channel[1] failed to connect");
+					exit(1);
+				}
+				checksock(channel);
+				rc = clientfunc(channel, count);
+				if (verbosity >= 1) {
+					cl_log(LOG_DEBUG, "%s[%d]%d: client %d ended rc:%d",
+					  procname, (int)getpid(), __LINE__, rc, i);
+				}
+				exit (rc > 127 ? 127 : rc);
+				break;
+
+			default:
+				break;
+		}
+	}
+
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d: server starting...",
+		  procname, (int)getpid(), __LINE__);
+	}
+	/* accept on server */
+	/* ***
+	 * Two problems (or more) here:
+	 * 1. What to do if no incoming call pending?
+	 *    At present, fudge by sleeping a little so client gets started.
+	 * 2. How to handle multiple clients?
+	 *    Would need to be able to await both new connections and
+	 *    data on existing connections.
+	 *    At present, fudge CLIENTS_MAX as 1.
+	 * ***
+	 */
+	sleep(1); /* *** */
+	channel = wconn->ops->accept_connection(wconn, NULL);
+	if (channel == NULL) {
+		cl_perror("server: acceptance failed");
+	}
+
+	checksock(channel);
+
+	rc = serverfunc(channel, count);
+
+	/* server finished: tidy up */
+	wconn->ops->destroy(wconn);
+
+	if (verbosity >= 1) {
+		cl_log(LOG_DEBUG, "%s[%d]%d: server ended rc:%d",
+		  procname, (int)getpid(), __LINE__, rc);
+	}
+
+	/* reap the clients */
+	for (i = 1; i <= clients; i++) {
+		pid_t pid;
+
+		pid = wait(&waitstat);
+		if (verbosity >= 1) {
+			cl_log(LOG_DEBUG, "%s[%d]%d: client %d reaped:%d",
+			  procname, (int)getpid(), __LINE__,
+			  (int) pid, WIFEXITED(waitstat));
+		}
+		if (WIFEXITED(waitstat)) {
+			rc += WEXITSTATUS(waitstat);
+		}else{
+			rc += 1;
+		}
+	}
+
+	return(rc);
 }
-#endif
+
+static void
+echomsgbody(void * body, int n, int niter, size_t * len)
+{
+	char *str = body;
+	int l;
+
+	l = snprintf(str, n-1, "String-%d", niter);
+	if (l < (n-1)) {
+		memset(&str[l], 'a', (n - (l+1)));
+	}
+	str[n-1] = '\0';
+	*len = n;
+}
+
 static void
 checkifblocked(IPC_Channel* chan)
 {
@@ -136,40 +367,103 @@ extern long	SeqNums[32];
 #endif
 
 static int
-transport_tests(int iterations)
+transport_tests(int iterations, int clients)
 {
 	int	rc = 0;
 
 #ifdef CHEAT_CHECKS
 	memset(SeqNums, 0, sizeof(SeqNums));
 #endif
-	rc += channelpair(echoclient, echoserver, iterations);
+	rc += (clients <= 0)
+	  ? channelpair(echoclient, echoserver, iterations)
+	  : clientserver(echoclient, echoserver, iterations, clients);
 
 #ifdef CHEAT_CHECKS
 	memset(SeqNums, 0, sizeof(SeqNums));
 #endif
-	rc += channelpair(asyn_echoclient, asyn_echoserver, iterations);
+	rc += (clients <= 0)
+	  ? channelpair(asyn_echoclient, asyn_echoserver, iterations)
+	  : clientserver(asyn_echoclient, asyn_echoserver, iterations, clients);
 
 #ifdef CHEAT_CHECKS
 	memset(SeqNums, 0, sizeof(SeqNums));
 #endif
-	rc += channelpair(mainloop_client, mainloop_server, iterations);
+	rc += (clients <= 0)
+	  ? channelpair(mainloop_client, mainloop_server, iterations)
+	  : clientserver(mainloop_client, mainloop_server, iterations, clients);
 
 	return rc;
 }
 
+int data_size = 10;
+
 int
 main(int argc, char ** argv)
 {
+	int argflag, argerrs;
+	int iterations;
+	int clients;
 	int	rc = 0;
 
+	/*
+	 * Check and process arguments.
+	 *	-v: verbose
+	 *	-i: number of iterations
+	 *	-c: number of clients (invokes client/server mechanism)
+	 *	-s: data-size
+	 */
+	procname = basename(argv[0]);
 
-	cl_log_set_entity("ipctest");
+	argerrs = 0;
+	iterations = iter_def;
+	clients = clients_def;
+	while ((argflag = getopt(argc, argv, "i:vuc:s:")) != EOF) {
+		switch (argflag) {
+		case 'i':	/* iterations */
+			iterations = atoi(optarg);
+			break;
+		case 'v':	/* verbosity */
+			verbosity++;
+			break;
+		case 'c':	/* number of clients */
+			clients = atoi(optarg);
+			if (clients < 1 || clients > CLIENTS_MAX) {
+				fprintf(stderr, "number of clients out of range"
+				  "(1 to %d)\n", CLIENTS_MAX);
+				argerrs++;
+			}
+			break;
+		case 's':	/* data size */
+			data_size = atoi(optarg);
+			if (data_size < 0) {
+				fprintf(stderr, "data size must be >=0\n");
+				argerrs++;
+			}
+			break;
+		default:
+			argerrs++;
+			break;
+		}
+	}
+	if (argerrs) {
+		fprintf(stderr,
+		  "Usage: %s [-v] [-i iterations] [-c clients] [-s size]\n"
+			"\t-v : verbose\n"
+			"\t-i : iterations (default %d)\n"
+			"\t-c : number of clients (default %d; nonzero invokes client/server)\n"
+			"\t-s : data size (default 10 bytes)\n",
+		  procname, iter_def, clients_def);
+		exit(1);
+	}
+
+	cl_malloc_forced_for_glib();
+
+	cl_log_set_entity(procname);
 	cl_log_enable_stderr(TRUE);
 
 
 
-	rc += transport_tests(10000);
+	rc += transport_tests(iterations, clients);
 
 #if 0
 	/* Broken for the moment - need to fix it long term */
@@ -178,7 +472,7 @@ main(int argc, char ** argv)
 	g_main_set_poll_func(cl_glibpoll);
 	ipc_set_pollfunc(cl_poll);
 
-	rc += transport_tests(50000);
+	rc += transport_tests(5 * iterations, clients);
 #endif
 	
 	cl_log(LOG_INFO, "TOTAL errors: %d", rc);
@@ -190,6 +484,10 @@ static int
 checksock(IPC_Channel* channel)
 {
 
+	if (!channel) {
+		cl_log(LOG_ERR, "Channel null");
+		return 1;
+	}
 	if (!IPC_ISRCONN(channel)) {
 		cl_log(LOG_ERR, "Channel status is %d"
 		", not IPC_CONNECT", channel->ch_status);
@@ -208,8 +506,8 @@ EOFcheck(IPC_Channel* chan)
 	cl_log(LOG_INFO, "channel state: %d", chan->ch_status);
 
 	if (chan->recv_queue->current_qlen > 0) {
-		cl_log(LOG_INFO, "EOF Receive queue has %d messages in it"
-		,	chan->recv_queue->current_qlen);
+		cl_log(LOG_INFO, "EOF Receive queue has %ld messages in it"
+		,	(long)chan->recv_queue->current_qlen);
 	}
 	if (fd <= 0) {
 		cl_log(LOG_INFO, "EOF receive fd: %d", fd);
@@ -259,24 +557,27 @@ EOFcheck(IPC_Channel* chan)
 static int
 echoserver(IPC_Channel* wchan, int repcount)
 {
-	char	str[256];
+	char	*str;
 	int	j;
 	int	errcount = 0;
 	IPC_Message	wmsg;
-	IPC_Message*	rmsg;
-
-
+	IPC_Message*	rmsg = NULL;
+	
+	str = malloc(data_size);
+	
+	memset(&wmsg, 0, sizeof(wmsg));
 	wmsg.msg_private = NULL;
 	wmsg.msg_done = NULL;
 	wmsg.msg_body = str;
+	wmsg.msg_buf = NULL;
 	wmsg.msg_ch = wchan;
 
 	cl_log(LOG_INFO, "Echo server: %d reps pid %d.", repcount, getpid());
 	for (j=1; j <= repcount
 	;++j, rmsg != NULL && (rmsg->msg_done(rmsg),1)) {
 		int	rc;
-		snprintf(str, sizeof(str)-1, "String-%d", j);
-		wmsg.msg_len = strlen(str)+1;
+
+		echomsgbody(str, data_size, j, &(wmsg.msg_len));
 		if ((rc = wchan->ops->send(wchan, &wmsg)) != IPC_OK) {
 			cl_log(LOG_ERR
 			,	"echotest: send failed %d rc iter %d"
@@ -342,6 +643,8 @@ echoserver(IPC_Channel* wchan, int repcount)
 #endif
 	wchan->ops->destroy(wchan); wchan = NULL;
 
+	free(str);
+
 	return errcount;
 }
 static int
@@ -378,6 +681,12 @@ echoclient(IPC_Channel* rchan, int repcount)
 			,	rc, j, errno);
 			cl_perror("recv");
 			++errcount;
+			if (errcount > MAXERRORS_RECV) {
+				cl_log(LOG_ERR,
+				  "echoclient: errcount excessive: %d: abandoning",
+				  errcount);
+				exit(1);
+			}
 			--j;
 			rmsg=NULL;
 			continue;
@@ -405,45 +714,6 @@ echoclient(IPC_Channel* rchan, int repcount)
 	return errcount;
 }
 
-static void
-echomsgbody(void * body, int niter, size_t * len)
-{
-	char *	str = body;
-	sprintf(str, "String-%d", niter);
-	*len = strlen(str)+1;
-}
-
-static void
-msg_free(IPC_Message* msg)
-{
-	
-#if 0
-	memset(msg->msg_body, 0xAA, msg->msg_len);
-#endif
-	free(msg->msg_body);
-#if 0
-	memset(msg, 0x55, sizeof(*msg));
-#endif
-	free(msg);
-
-}
-
-static IPC_Message*
-newmessage(IPC_Channel* chan, int niter)
-{
-	IPC_Message*	msg;
-
-	msg = malloc(sizeof(*msg));
-	if (msg) {
-		msg->msg_private = NULL;
-		msg->msg_done = msg_free;
-		msg->msg_ch = chan;
-		msg->msg_body = malloc(32);
-		echomsgbody(msg->msg_body, niter, &msg->msg_len);
-	}
-	return msg;
-}
-
 void dump_ipc_info(IPC_Channel* chan);
 
 static int
@@ -501,7 +771,34 @@ checkinput(IPC_Channel* chan, const char * where, int* rdcount, int maxcount)
 	return errs;
 }
 
+static void
+async_high_flow_callback(IPC_Channel* ch, void* userdata)
+{
+	int* stopsending = userdata;
+	
+	if (userdata == NULL){
+		cl_log(LOG_ERR, "userdata is NULL");
+		return;
+	}
+	
+	*stopsending = 1;
+	
+}
 
+static void
+async_low_flow_callback(IPC_Channel* ch, void* userdata)
+{
+
+	int* stopsending = userdata;
+	
+	if (userdata == NULL){
+		cl_log(LOG_ERR, "userdata is NULL");
+		return;
+	}
+
+	*stopsending = 0;
+	
+}
 
 
 static int
@@ -512,52 +809,73 @@ asyn_echoserver(IPC_Channel* wchan, int repcount)
 	int		errcount = 0;
 	int		blockedcount = 0;
 	IPC_Message*	wmsg;
-	int		lastcount = -1;
 	const char*	w = "asyn_echoserver";
-
-
+	int		stopsending = 0;
 
 	cl_log(LOG_INFO, "Asyn echo server: %d reps pid %d."
 	,	repcount, (int)getpid());
+	
+	(void)async_high_flow_callback;
+	(void)async_low_flow_callback;
+	
+	
+	wchan->ops->set_high_flow_callback(wchan, async_high_flow_callback, &stopsending);
+	wchan->ops->set_low_flow_callback(wchan, async_low_flow_callback, &stopsending);
+	  
+	wchan->low_flow_mark = 2;
+	wchan->high_flow_mark = 20;
+	
 	while (rdcount < repcount) {
 		int	rc;
-
-		do {
-			++wrcount;
-			if (wrcount > repcount) {
-				break;
-			}
-			wmsg = newmessage(wchan, wrcount);
-
-			/*fprintf(stderr, "s"); */
-			if ((rc = wchan->ops->send(wchan, wmsg)) != IPC_OK) {
-				cl_log(LOG_ERR
-				,	"asyn_echoserver: send failed"
-				" %d rc iter %d"
-				,	rc, wrcount);
-				++errcount;
-				continue;
-			}
-			lastcount = wrcount;
+		
+		while (wrcount < repcount && blockedcount < 10
+		       && wchan->ch_status != IPC_DISCONNECT 
+		       ){
 			
-			if (wchan->ops->is_sending_blocked(wchan)) {
-				/* fprintf(stderr, "b"); */
-				++blockedcount;
-			}else{
-				blockedcount = 0;
+			if (!stopsending){
+				++wrcount;
+				if (wrcount > repcount) {
+					break;
+				}
+				wmsg = wchan->ops->new_ipcmsg(wchan, NULL, data_size, NULL);
+				echomsgbody(wmsg->msg_body, data_size, wrcount, &wmsg->msg_len);
+				if ((rc = wchan->ops->send(wchan, wmsg)) != IPC_OK){
+					
+					cl_log(LOG_INFO, "channel sstatus in echo server is %d",
+					       wchan->ch_status);
+					if (wchan->ch_status != IPC_CONNECT) {
+						cl_log(LOG_ERR
+						       ,	"asyn_echoserver: send failed"
+						       " %d rc iter %d"
+						       ,	rc, wrcount);
+						++errcount;
+						continue;
+					}else {/*send failed because of channel busy
+						* roll back
+						*/
+						--wrcount;
+					}				
+				}
+				
+				if (wchan->ops->is_sending_blocked(wchan)) {
+					/* fprintf(stderr, "b"); */
+					++blockedcount;
+				}else{
+					blockedcount = 0;
+				}
 			}
+			
+			
 			errcount += checkinput(wchan, w, &rdcount, repcount);
 			if (wrcount < repcount
-			&&	wchan->ch_status == IPC_DISCONNECT) {
+			    &&	wchan->ch_status == IPC_DISCONNECT) {
 				++errcount;
 				break;
 			}
-		}while (wrcount < repcount && blockedcount < 10
-		&&	wchan->ch_status != IPC_DISCONNECT);
-
-		if (wrcount < repcount) {
-			/* fprintf(stderr, "B"); */
 		}
+		
+/*  		cl_log(LOG_INFO, "async_echoserver: wrcount =%d rdcount=%d B", wrcount, rdcount); */
+
 		wchan->ops->waitout(wchan);
 		errcount += checkinput(wchan, w, &rdcount, repcount);
 		if (wrcount >= repcount && rdcount < repcount) {
@@ -565,28 +883,30 @@ asyn_echoserver(IPC_Channel* wchan, int repcount)
 			
 			if (rc != IPC_OK) {
 				cl_log(LOG_ERR
-				,	"asyn_echoserver: waitin()"
-				" failed %d rc rdcount %d errno=%d"
+				       ,	"asyn_echoserver: waitin()"
+				       " failed %d rc rdcount %d errno=%d"
 				,	rc, rdcount, errno);
 				cl_perror("waitin");
 				exit(1);
 			}
 		}
 		if (wchan->ch_status == IPC_DISCONNECT
-		&&	rdcount < repcount) {
-			cl_log(LOG_ERR
-			,	"asyn_echoserver: EOF in iter %d"
-			,	rdcount);
+		    &&	rdcount < repcount) {
+			cl_log(LOG_ERR,
+			       "asyn_echoserver: EOF in iter %d (wrcount=%d)",
+			       rdcount, wrcount);
 			EOFcheck(wchan);
 			++errcount;
 			break;
 		}
 
+		blockedcount = 0;
+
 	}
 
 	cl_log(LOG_INFO, "asyn_echoserver: %d errors", errcount);
 #if 0
-	cl_log(LOG_INFO, "destroying channel 0x%lx", (unsigned long)wchan);
+	cl_log(LOG_INFO, "%d destroying channel 0x%lx", getpid(), (unsigned long)wchan);
 #endif
 	wchan->ops->destroy(wchan); wchan = NULL;
 	return errcount;
@@ -655,26 +975,34 @@ asyn_echoclient(IPC_Channel* chan, int repcount)
 			}
 			/*fprintf(stderr, "c"); */
 			++rdcount;
-			if ((rc = chan->ops->send(chan, rmsg))
-			!=	IPC_OK) {
+
+			
+			do {
+				rc = chan->ops->send(chan, rmsg);
+				
+			}while (rc != IPC_OK && chan->ch_status == IPC_CONNECT);
+
+			if (chan->ch_status !=  IPC_CONNECT){
 				++errcount;
 				cl_perror("send");
 				cl_log(LOG_ERR
-				,	"Async echoclient: send failed"
-				" rc %d, iter %d", rc, rdcount);
+				       ,	"Async echoclient: send failed"
+				       " rc %d, iter %d", rc, rdcount);
 				cl_log(LOG_INFO, "Message being sent: %s"
-				,		(char*)rmsg->msg_body);
+				       ,		(char*)rmsg->msg_body);
 				if (!IPC_ISRCONN(chan)) {
 					cl_log(LOG_ERR
-					,	"Async echoclient: EOF(2)"
-					" iter %d", rdcount+1);
+					       ,	"Async echoclient: EOF(2)"
+					       " iter %d", rdcount+1);
 					EOFcheck(chan);
 					return errcount;
 				}
 				continue;
-			}else{
-				++wrcount;
+				
 			}
+
+			
+			++wrcount;
 			/*fprintf(stderr, "x"); */
 		}
 		if (rdcount >= repcount) {
@@ -752,11 +1080,15 @@ asyn_echoclient(IPC_Channel* chan, int repcount)
 	}
 	cl_poll_ignore(rfd);
 	cl_poll_ignore(wfd);
-	cl_log(LOG_INFO, "Async echoclient: %d errors, %d reads, %d writes"
-	,	errcount, rdcount, wrcount);
+	cl_log(LOG_INFO, "Async echoclient: %d errors, %d reads, %d writes",
+	       errcount, rdcount, wrcount);
 #if 0
-	cl_log(LOG_INFO, "destroying channel 0x%lx", (unsigned long)chan);
+	cl_log(LOG_INFO, "%d destroying channel 0x%lx",getpid(), (unsigned long)chan);
 #endif
+
+	
+	chan->ops->waitout(chan);
+
 	chan->ops->destroy(chan); chan = NULL;
 	return errcount;
 }
@@ -773,35 +1105,32 @@ struct iterinfo {
 
 static GMainLoop*	loop = NULL;
 
+
+
+
 static gboolean
 s_send_msg(gpointer data)
 {
 	struct iterinfo*i = data;
 	IPC_Message*	wmsg;
-	int		rc;
+	int		rc;	
 	
-	/* Flow control? */
-	if (i->chan->send_queue->current_qlen
-	>=	i->chan->send_queue->max_qlen-2) {
-		i->sendingsuspended = TRUE;
-		return FALSE;
-	}
-	if (i->sendingsuspended) {
-		i->sendingsuspended = FALSE;
-	}
 	++i->wcount;
 	
-	wmsg = newmessage(i->chan, i->wcount);
-	/*fprintf(stderr, "s");*/
+	wmsg = i->chan->ops->new_ipcmsg(i->chan, NULL, data_size, NULL);
+	echomsgbody(wmsg->msg_body, data_size, i->wcount, &wmsg->msg_len);
+	
+	/*cl_log(LOG_INFO, "s_send_msg: sending out %d", i->wcount);*/
+	
 	if ((rc = i->chan->ops->send(i->chan, wmsg)) != IPC_OK) {
 		cl_log(LOG_ERR
 		,	"s_send_msg: send failed"
 		" %d rc iter %d"
 		,	rc, i->wcount);
 		cl_log(LOG_ERR
-		,	"s_send_msg: channel status: %d qlen: %d"
+		,	"s_send_msg: channel status: %d qlen: %ld"
 		,	i->chan->ch_status
-		,	i->chan->send_queue->current_qlen);
+		,	(long)i->chan->send_queue->current_qlen);
 		++i->errcount;
 		if (i->chan->ch_status != IPC_CONNECT) {
 			cl_log(LOG_ERR,	"s_send_msg: Exiting.");
@@ -812,7 +1141,44 @@ s_send_msg(gpointer data)
 			return FALSE;
 		}
 	}
-	return i->wcount < i->max;
+	return !i->sendingsuspended?i->wcount < i->max: FALSE;
+}
+
+
+
+
+static void
+mainloop_low_flow_callback(IPC_Channel* ch, void* userdata)
+{
+	
+	struct iterinfo* i = (struct iterinfo*) userdata;
+	
+	if (userdata == NULL){
+		cl_log(LOG_ERR, "userdata is NULL");
+		return;
+	}
+	
+	if (i->sendingsuspended){
+		i->sendingsuspended = FALSE;
+		g_idle_add(s_send_msg, i);
+	}
+	
+	return;
+	
+}
+
+static void
+mainloop_high_flow_callback(IPC_Channel* ch, void* userdata)
+{
+	struct iterinfo* i = (struct iterinfo*) userdata;
+	
+	if (userdata == NULL){
+		cl_log(LOG_ERR, "userdata is NULL");
+		return;
+	}
+	
+	i->sendingsuspended = TRUE;
+	
 }
 
 
@@ -822,13 +1188,7 @@ s_rcv_msg(IPC_Channel* chan, gpointer data)
 	struct iterinfo*i = data;
 
 	i->errcount += checkinput(chan, "s_rcv_msg", &i->rcount, i->max);
-
-	if (i->sendingsuspended
-	&&	!chan->ops->is_sending_blocked(chan)) {
-		i->sendingsuspended = FALSE;
-		g_idle_add(s_send_msg, data);
-	}
-
+	
 	if (chan->ch_status == IPC_DISCONNECT
 	||	i->rcount >= i->max || i->errcount > MAXERRORS) {
 		if (i->rcount < i->max) {
@@ -845,10 +1205,12 @@ s_rcv_msg(IPC_Channel* chan, gpointer data)
 static gboolean
 checkmsg(IPC_Message* rmsg, const char * who, int rcount)
 {
-	char		str[256];
+	char		*str;
 	size_t		len;
+	
+	str = malloc(data_size);
 
-	echomsgbody(str, rcount, &len);
+	echomsgbody(str, data_size, rcount, &len);
 
 	if (rmsg->msg_len != len) {
 		cl_log(LOG_ERR
@@ -889,6 +1251,9 @@ checkmsg(IPC_Message* rmsg, const char * who, int rcount)
 		,	who, rcount);
 #endif
 	}
+
+	free(str);
+
 	return TRUE;
 }
 
@@ -903,14 +1268,7 @@ s_echo_msg(IPC_Channel* chan, gpointer data)
 		if (chan->ch_status == IPC_DISCONNECT) {
 			break;
 		}
-		if (i->chan->send_queue->current_qlen
-		>=	i->chan->send_queue->max_qlen-2) {
-			i->sendingsuspended = TRUE;
-			cl_log(LOG_INFO
-			,	"s_echo_msg: Sending suspended.");
-			goto retout;
-		}
-		i->sendingsuspended = FALSE;
+
 		if ((rc = chan->ops->recv(chan, &rmsg)) != IPC_OK) {
 			cl_log(LOG_ERR
 			,	"s_echo_msg: recv failed %d rc iter %d"
@@ -925,21 +1283,32 @@ s_echo_msg(IPC_Channel* chan, gpointer data)
 			++i->errcount;
 		}
 
-		/*fprintf(stderr, "c");*/
-		if ((rc = chan->ops->send(chan, rmsg)) != IPC_OK) {
-			cl_log(LOG_ERR
-			,	"s_echo_msg: send failed %d rc iter %d qlen %d"
-			,	rc, i->rcount, chan->send_queue->current_qlen);
-			cl_perror("s_echo_msg:send");
-			++i->errcount;
-		}else{
-			i->wcount+=1;
+		
+		
+		/*cl_log(LOG_INFO, "s_echo_msg: rcount= %d, wcount =%d", i->rcount, i->wcount);*/
+		
+		
+		do {
+			rc = chan->ops->send(chan, rmsg);
+			
+		}while (rc != IPC_OK && chan->ch_status == IPC_CONNECT);
+		
+		if (chan->ch_status !=  IPC_CONNECT){
+			cl_log(LOG_ERR,
+			       "s_echo_msg: send failed %d rc iter %d qlen %ld",
+			       rc, i->rcount, (long)chan->send_queue->current_qlen);
+			cl_perror("send");
+			i->errcount ++;
+			
 		}
+		
+		i->wcount+=1;
+		/*cl_log(LOG_INFO, "s_echo_msg: end of this ite");*/
 	}
-retout:
+ retout:
 	/*fprintf(stderr, "%%");*/
 	if (i->rcount >= i->max || chan->ch_status == IPC_DISCONNECT
-	||	i->errcount > MAXERRORS) {
+	    ||	i->errcount > MAXERRORS) {
 		chan->ops->waitout(chan);
 		g_main_quit(loop);
 		return FALSE;
@@ -953,6 +1322,7 @@ init_iterinfo(struct iterinfo * i, IPC_Channel* chan, int max)
 	memset(i, 0, sizeof(*i));
 	i->chan = chan;
 	i->max = max;
+	i->sendingsuspended = FALSE;
 }
 
 static int
@@ -961,8 +1331,17 @@ mainloop_server(IPC_Channel* chan, int repcount)
 	struct iterinfo info;
 	GCHSource*	msgchan;
 	guint		sendmsgsrc;
+
+	
+
 	loop = g_main_new(FALSE);
 	init_iterinfo(&info, chan, repcount);
+
+	chan->ops->set_high_flow_callback(chan, mainloop_high_flow_callback, &info);
+	chan->ops->set_low_flow_callback(chan, mainloop_low_flow_callback, &info);
+	chan->high_flow_mark = 20;
+	chan->low_flow_mark = 2;
+
 	sendmsgsrc = g_idle_add(s_send_msg, &info);
 	msgchan = G_main_add_IPC_Channel(G_PRIORITY_DEFAULT, chan
 	,	FALSE, s_rcv_msg, &info, NULL);
@@ -982,7 +1361,7 @@ mainloop_client(IPC_Channel* chan, int repcount)
 	init_iterinfo(&info, chan, repcount);
 	G_main_add_IPC_Channel(G_PRIORITY_DEFAULT, chan
 	,	FALSE, s_echo_msg, &info, NULL);
-	cl_log(LOG_INFO, "Mainloop echo client: %d reps pid %d.", repcount, (int)getpid());
+	cl_log(LOG_INFO, "Mainloop echo client: %d reps pid %d.", repcount, (int)getpid());	
 	g_main_run(loop);
 	g_main_destroy(loop);
 	loop = NULL;
