@@ -1,4 +1,3 @@
-/* $Id: proctrack.c,v 1.16.2.1 2004/04/20 08:39:45 alan Exp $ */
 /*
  * Process tracking object.
  *
@@ -21,7 +20,7 @@
  *
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <errno.h>
 #include <sys/wait.h>
 #include <heartbeat.h>
@@ -29,6 +28,8 @@
 #include <signal.h>
 #include <clplumbing/proctrack.h>
 #include <clplumbing/cl_log.h>
+#include <clplumbing/uids.h>
+#include <clplumbing/cl_signal.h>
 #include <clplumbing/Gmain_timeout.h>
 
 #define	DEBUGPROCTRACK	debugproctrack
@@ -79,6 +80,133 @@ NewTrackedProc(pid_t pid, int isapgrp, ProcTrackLogType loglevel
 	}
 }
 
+static struct signal_info_s {
+	int		signo;
+	const char *	sigdefine;
+	const char*	sigwords;
+} signal_info [] = {
+
+#ifdef SIGHUP
+	{SIGHUP,	"SIGHUP",		"Hangup"},
+#endif
+#ifdef SIGINT
+	{SIGINT,	"SIGINT",		"Interrupt"},
+#endif
+#ifdef SIGQUIT
+	{SIGQUIT,	"SIGQUIT",		"Quit"},
+#endif
+#ifdef SIGILL
+	{SIGILL,	"SIGILL",		"Illegal instruction"},
+#endif
+#ifdef SIGTRAP
+	{SIGTRAP,	"SIGTRAP",		"Trace"},
+#endif
+#ifdef SIGABRT
+	{SIGABRT,	"SIGABRT",		"Abort"},
+#endif
+#ifdef SIGIOT
+	{SIGIOT,	"SIGIOT",		"IOT trap"},
+#endif
+#ifdef SIGBUS
+	{SIGBUS,	"SIGBUS",		"BUS error"},
+#endif
+#ifdef SIGFPE
+	{SIGFPE,	"SIGFPE",		"Floating-point exception"},
+#endif
+#ifdef SIGKILL
+	{SIGKILL,	"SIGKILL",		"Kill, unblockable"},
+#endif
+#ifdef SIGUSR1
+	{SIGUSR1,	"SIGUSR1",		"User-defined signal 1"},
+#endif
+#ifdef SIGSEGV
+	{SIGSEGV,	"SIGSEGV",		"Segmentation violation"},
+#endif
+#ifdef SIGUSR2
+	{SIGUSR2,	"SIGUSR2",		"User-defined signal 2"},
+#endif
+#ifdef SIGPIPE
+	{SIGPIPE,	"SIGPIPE",		"Broken pipe (POSIX)"},
+#endif
+#ifdef SIGALRM
+	{SIGALRM,	"SIGALRM",		"Alarm clock (POSIX)"},
+#endif
+#ifdef SIGTERM
+	{SIGTERM,	"SIGTERM",		"Termination (ANSI)"},
+#endif
+#ifdef SIGSTKFLT
+	{SIGSTKFLT,	"SIGSTKFLT",		"Stack fault"},
+#endif
+#ifdef SIGCHLD
+	{SIGCHLD,	"SIGCHLD",		"Child status has changed"},
+#endif
+#ifdef SIGCLD
+	{SIGCLD,	"SIGCLD	",		"Child status has changed"},
+#endif
+#ifdef SIGCONT
+	{SIGCONT,	"SIGCONT",		"Continue"},
+#endif
+#ifdef SIGSTOP
+	{SIGSTOP,	"SIGSTOP",		"Stop, unblockable"},
+#endif
+#ifdef SIGTSTP
+	{SIGTSTP,	"SIGTSTP",		"Keyboard stop"},
+#endif
+#ifdef SIGTTIN
+	{SIGTTIN,	"SIGTTIN",		"Background read from tty"},
+#endif
+#ifdef SIGTTOU
+	{SIGTTOU,	"SIGTTOU",		"Background write to tty"},
+#endif
+#ifdef SIGURG
+	{SIGURG,	"SIGURG	",		"Urgent condition on socket"},
+#endif
+#ifdef SIGXCPU
+	{SIGXCPU,	"SIGXCPU",		"CPU limit exceeded"},
+#endif
+#ifdef SIGXFSZ
+	{SIGXFSZ,	"SIGXFSZ",		"File size limit exceeded"},
+#endif
+#ifdef SIGVTALRM
+	{SIGVTALRM,	"SIGVTALRM",		"Virtual alarm clock"},
+#endif
+#ifdef SIGPROF
+	{SIGPROF,	"SIGPROF",		"Profiling alarm clock"},
+#endif
+#ifdef SIGWINCH
+	{SIGWINCH,	"SIGWINCH",		"Window size change"},
+#endif
+#ifdef SIGPOLL
+	{SIGPOLL,	"SIGPOLL",		"Pollable event occurred"},
+#endif
+#ifdef SIGIO
+	{SIGIO,		"SIGIO",		"I/O now possible"},
+#endif
+#ifdef SIGPWR
+	{SIGPWR,	"SIGPWR",		"Power failure restart"},
+#endif
+#ifdef SIGSYS
+	{SIGSYS,	"SIGSYS",		"Bad system call"},
+#endif
+};
+static const char *
+signal_name(int signo, const char ** sigdescription)
+{
+	int	j;
+	for (j=0; j < DIMOF(signal_info); ++j) {
+		if (signal_info[j].signo == signo) {
+			if (sigdescription) {
+				*sigdescription = signal_info[j].sigwords;
+			}
+			return signal_info[j].sigdefine;
+		}
+	}
+	if (sigdescription) {
+		*sigdescription = NULL;
+	}
+	return NULL;
+}
+
 /* returns TRUE if 'pid' was registered */
 int
 ReportProcHasDied(int pid, int status)
@@ -116,13 +244,6 @@ ReportProcHasDied(int pid, int status)
 		signo = WTERMSIG(status);
 		doreport=1;
 	}
-#ifdef WCOREDUMP
-	if (WCOREDUMP(status)) {
-		/* Force a report on all core dumping processes */
-		didcoredump=1;
-		doreport=1;
-	}
-#endif
 	switch(level) {
 		case PT_LOGVERBOSE:	doreport=1;
 					break;
@@ -136,6 +257,13 @@ ReportProcHasDied(int pid, int status)
 	if (!LoggingIsEnabled) {
 		doreport = 0;
 	}
+#ifdef WCOREDUMP
+	if (WCOREDUMP(status)) {
+		/* Force a report on all core dumping processes */
+		didcoredump=1;
+		doreport=1;
+	}
+#endif
 	if (DEBUGPROCTRACK && !doreport) {
 		doreport = 1;
 		debugreporting = 1;
@@ -144,18 +272,31 @@ ReportProcHasDied(int pid, int status)
 	if (doreport) {
 		if (deathbyexit) {
 			cl_log((exitcode == 0 ? LOG_INFO : LOG_WARNING)
-			,	"Exiting %s process %d returned rc %d."
+			,	"Managed %s process %d exited with return code %d."
 			,	type, pid, exitcode);
 		}else if (deathbysig) {
+			const char *	signame = NULL;
+			const char *	sigwords = NULL;
+			int		logtype;
+			signame = signal_name(signo, &sigwords);
+			logtype = (debugreporting ? LOG_INFO : LOG_WARNING);
 			/*
 			 * Processes being killed isn't an error if
 			 * we're only logging because of debugging.
 			 */
-			cl_log((debugreporting ? LOG_DEBUG : LOG_ERR)
-			,	"Exiting %s process %d killed by signal %d."
-			,	type, pid, signo);
+			if (signame && sigwords) {
+				cl_log(logtype
+				,	"Managed %s process %d killed by"
+				" signal %d [%s - %s]."
+				,	type, pid, signo
+				,	signame, sigwords);
+			}else{
+				cl_log(logtype
+				,	"Managed %s process %d killed by signal %d."
+				,	type, pid, signo);
+			}
 		}else{
-			cl_log(LOG_ERR, "Exiting %s process %d went away"
+			cl_log(LOG_ERR, "Managed %s process %d went away"
 			" strangely (!)"
 			,	type, pid);
 		}
@@ -163,16 +304,13 @@ ReportProcHasDied(int pid, int status)
 #ifdef WCOREDUMP
 	if (didcoredump) {
 		/* We report ALL core dumps without exception */
-		cl_log(LOG_ERR, "Exiting %s process %d dumped core"
+		cl_log(LOG_ERR, "Managed %s process %d dumped core"
 		,	type, pid);
 	}
 #endif
 
 	if (p) {
-		if (p->timerid > 0) {
-			g_source_remove(p->timerid);
-			p->timerid = 0;
-		}
+		RemoveTrackedProcTimeouts(pid);
 		/*
 		 * From clplumbing/proctrack.h:
 		 * (ProcTrack* p, int status, int signo, int exitcode
@@ -181,7 +319,7 @@ ReportProcHasDied(int pid, int status)
 		p->ops->procdied(p, status, signo, exitcode,  doreport);
 		if (p->privatedata) {
 			/* They may have forgotten to free something... */
-			cl_log(LOG_ERR, "Exiting %s process %d did not"
+			cl_log(LOG_ERR, "Managed %s process %d did not"
 			" clean up private data!"
 			,	type, pid);
 		}
@@ -222,44 +360,108 @@ SetTrackedProcTimeouts(pid_t pid, ProcTrackKillInfo* info)
 	return pinfo->timerid;
 }
 
+void
+RemoveTrackedProcTimeouts(pid_t pid)
+{
+	ProcTrack*	pinfo;
+	pinfo = GetProcInfo(pid);
+	
+	if (pinfo == NULL) {
+		return;
+	}
+
+	if (pinfo->killinfo && pinfo->timerid) {
+		Gmain_timeout_remove(pinfo->timerid);
+	}
+	pinfo->killinfo = NULL;
+	pinfo->timerid = 0;
+}
+
 static gboolean
 TrackedProcTimeoutFunction(gpointer p)
 {
-	pid_t		pid = GPOINTER_TO_INT(p);
+	/* This is safe - Pids are relatively small ints */
+	pid_t		pid = POINTER_TO_SIZE_T(p); /*pointer cast as int*/
 	ProcTrack*	pinfo;
 	int		nsig;
 	long		mstimeout;
+	int		hadprivs;
 
 	pinfo = GetProcInfo(pid);
 	
-	if (pinfo == NULL || pinfo->timeoutseq < 0
-	||	pinfo->killinfo == NULL) {
+	if (pinfo == NULL) {
+		cl_log(LOG_ERR, "%s: bad pinfo in call (pid %d)", __FUNCTION__, pid);
+		return FALSE;
+	}
+	if (pinfo->timeoutseq < 0 || pinfo->killinfo == NULL) {
+		cl_log(LOG_ERR
+		,	 "%s: bad call (pid %d): killinfo (%d, 0x%lx)"
+		,	__FUNCTION__, pid
+		,	pinfo->timeoutseq
+		,	(unsigned long)POINTER_TO_SIZE_T(pinfo->killinfo));
 		return FALSE;
 	}
 
 	pinfo->timerid = 0;
 	nsig = pinfo->killinfo[pinfo->timeoutseq].signalno;
-	mstimeout = pinfo->killinfo[pinfo->timeoutseq].mstimeout;
 
 	if (nsig == 0) {
+		if (CL_PID_EXISTS(pid)) {
+			cl_log(LOG_ERR
+			,	"%s: %s process (PID %d) will not die!"
+			,	__FUNCTION__
+			,	pinfo->ops->proctype(pinfo)
+			,	(int)pid);
+		}
 		return FALSE;
 	}
-	cl_log(LOG_WARNING, "%s process (PID %d) timed out"
-	".  Killing with signal %d."
-	,	pinfo->ops->proctype(pinfo), (int)pid, nsig);
+	pinfo->timeoutseq++;
+	cl_log(LOG_WARNING, "%s process (PID %d) timed out (try %d)"
+	".  Killing with signal %s (%d)."
+	,	pinfo->ops->proctype(pinfo), (int)pid
+	,	pinfo->timeoutseq
+	,	signal_name(nsig, NULL)
+	,	nsig);
 
+	if (pinfo->isapgrp && nsig > 0) {
+		pid = -pid;
+	}
+
+	if (!(hadprivs = cl_have_full_privs())) {
+		return_to_orig_privs();
+	}
 	if (kill(pid, nsig) < 0) {
-		if (errno == EEXIST) {
-			/* No point in trying this again ;-) */
+		if (errno == ESRCH) {
+			/* Mission accomplished! */
+		cl_log(LOG_INFO, "%s process (PID %d) died before killing (try %d)"
+		,	pinfo->ops->proctype(pinfo), (int)pid
+		,	pinfo->timeoutseq);
 			return FALSE;
 		}else{
-			cl_perror("kill(%d,%d) failed"
-			,	pid, nsig);
+			cl_perror("%s: kill(%d,%d) failed"
+			,	__FUNCTION__, pid, nsig);
 		}
 	}
+	if (!hadprivs) {
+		return_to_dropped_privs();
+	}
+	mstimeout = pinfo->killinfo[pinfo->timeoutseq].mstimeout;
 	pinfo->timerid = Gmain_timeout_add(mstimeout
 	,	TrackedProcTimeoutFunction
-	,	GINT_TO_POINTER(pid));
+	,	p);
+	if (pinfo->timerid <= 0) {
+		cl_log(LOG_ERR, "%s: Could not add new kill timer [%u]"
+		,	__FUNCTION__, pinfo->timerid);
+		kill(pid, SIGKILL);
+	}
+	if (debugproctrack) {
+		cl_log(LOG_DEBUG, "%s process (PID %d) scheduled to be killed again"
+		" (try %d) in %ld ms [timerid %u]"
+		,	pinfo->ops->proctype(pinfo), (int)pid
+		,	pinfo->timeoutseq
+		,	mstimeout
+		,	pinfo->timerid);
+	}
 	return FALSE;
 }
 
