@@ -161,6 +161,7 @@ static int		get_iflist(llc_private_t*, const char *host);
 static void		zap_iflist(llc_private_t*);
 static void		zap_order_seq(llc_private_t* pi);
 static void		zap_order_queue(llc_private_t* pi);
+static void		zap_msg_queue(llc_private_t* pi);
 static int		enqueue_msg(llc_private_t*,struct ha_msg*);
 static struct ha_msg*	dequeue_msg(llc_private_t*);
 static gen_callback_t*	search_gen_callback(const char * type, llc_private_t*);
@@ -361,8 +362,9 @@ hb_api_signon(struct ll_cluster* cinfo, const char * clientid)
 
 	/* Connect to the heartbeat API server */
 
-	if ((pi->chan = ipc_channel_constructor(IPC_ANYTYPE, wchanattrs))
-	==	NULL) {
+	pi->chan = ipc_channel_constructor(IPC_ANYTYPE, wchanattrs);
+	g_hash_table_destroy(wchanattrs);
+	if (pi->chan == NULL) {
 		ha_api_log(LOG_ERR, "hb_api_signon: Can't connect"
 		" to heartbeat");
 		ZAPMSG(request);
@@ -370,6 +372,7 @@ hb_api_signon(struct ll_cluster* cinfo, const char * clientid)
 	}
 
 	pi->chan->should_send_block = TRUE;
+	pi->chan->refcount++;
 
         if (pi->chan->ops->initiate_connection(pi->chan) != IPC_OK) {
 		ha_api_log(LOG_ERR, "hb_api_signon: Can't initiate"
@@ -401,7 +404,7 @@ hb_api_signon(struct ll_cluster* cinfo, const char * clientid)
 	if ((result = ha_msg_value(reply, F_APIRESULT)) != NULL
 	&&	strcmp(result, API_OK) == 0) {
 		rc = HA_OK;
-		pi->SignedOn = 1;
+		pi->SignedOn = TRUE;
 
 		if ((tmpstr = ha_msg_value(reply, F_DEADTIME)) == NULL
 		||	sscanf(tmpstr, "%lx", (unsigned long*)&(pi->deadtime_ms)) != 1) {
@@ -455,33 +458,44 @@ hb_api_signoff(struct ll_cluster* cinfo,gboolean need_destroy_chan)
 		return HA_FAIL;
 	}
 	pi = (llc_private_t*)cinfo->ll_cluster_private;
-
-	if (!pi->SignedOn) {
-		/* Already signed off... No value in returning an error... */
-		return HA_OK;
+	if (debug_level > 1) {
+		cl_log(LOG_DEBUG, "%s(%d){"
+		,	__FUNCTION__, need_destroy_chan);
 	}
 
-	if ((request = hb_api_boilerplate(API_SIGNOFF)) == NULL) {
-		ha_api_log(LOG_ERR, "hb_api_signoff: can't create msg");
-		return HA_FAIL;
-	}
-	
-	/* Send the message */
-	if (msg2ipcchan(request, pi->chan) != HA_OK) {
+	if (pi->SignedOn && pi->chan && IPC_ISWCONN(pi->chan)) {
+		if ((request = hb_api_boilerplate(API_SIGNOFF)) == NULL) {
+			ha_api_log(LOG_ERR, "hb_api_signoff: can't create msg");
+			return HA_FAIL;
+		}
+		
+		/* Send the message */
+		if (msg2ipcchan(request, pi->chan) != HA_OK) {
+			ZAPMSG(request);
+			ha_api_perror("can't send message to IPC");
+			return HA_FAIL;
+		}
+		pi->chan->ops->waitout(pi->chan);
 		ZAPMSG(request);
-		ha_api_perror("can't send message to IPC");
-		return HA_FAIL;
 	}
-	pi->chan->ops->waitout(pi->chan);
-	ZAPMSG(request);
 	OurClientID[0] = EOS;
-	if(need_destroy_chan){
-		pi->chan->ops->destroy(pi->chan);
+	
+	if(pi->chan) {
+		if (need_destroy_chan) {
+			pi->chan->ops->destroy(pi->chan);
+			pi->chan = NULL;
+		} else if (IPC_ISRCONN(pi->chan)) {
+			pi->chan->ops->disconnect(pi->chan);
+		}
 	}
-	pi->SignedOn = 0;
+	pi->SignedOn = FALSE;
 	zap_order_seq(pi);
 	zap_order_queue(pi);
 
+	if (debug_level > 1) {
+		cl_log(LOG_DEBUG, "}/*%s(%d)*/", __FUNCTION__
+		,	need_destroy_chan);
+	}
 	return HA_OK;
 }
 /*
@@ -504,7 +518,8 @@ hb_api_delete(struct ll_cluster* ci)
 	zap_iflist(pi);
 	zap_nodelist(pi);
 
-	/* What about our message queue? */
+	/* Free up the message queue */
+	zap_msg_queue(pi);
 
 	/* Free up the private information */
 	memset(pi, 0, sizeof(*pi));
@@ -1480,6 +1495,23 @@ zap_order_queue(llc_private_t* pi)
 	pi->order_queue_head = NULL;
 }
 
+static void
+zap_msg_queue(llc_private_t* pi)
+{
+	struct MsgQueue* qelem = pi->firstQdmsg;
+	struct MsgQueue* next;
+
+	while (qelem != NULL){
+		next = qelem->next;
+		ZAPMSG(qelem->value);
+		cl_free(qelem);
+		qelem = next;	 
+	}
+	pi->firstQdmsg = NULL;
+	pi->lastQdmsg = NULL;
+}
+
+
 /*
  * Create a new stringlist.
  */
