@@ -1,4 +1,3 @@
-/* $Id: bcast.c,v 1.33.2.3 2004/09/11 06:36:33 msoffen Exp $ */
 /*
  * bcast.c: UDP/IP broadcast-based communication code for heartbeat.
  *
@@ -23,7 +22,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -80,31 +79,6 @@ static int		localudpport = -1;
 
 int if_get_broadaddr(const char *ifn, struct in_addr *broadaddr);
 
-/*
- * bcastclose is called as part of unloading the bcast HBcomm plugin.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the plugin, and not a single interface
- * in particular, here's our chance to clean it up.
- */
-
-static void
-bcastclosepi(PILPlugin*pi)
-{
-}
-
-
-/*
- * bcastcloseintf called as part of shutting down the bcast HBcomm interface.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the bcast implementation, here's our chance
- * to clean it up.
- */
-static PIL_rc
-bcastcloseintf(PILInterface* pi, void* pd)
-{
-	return PIL_OK;
-}
-
 static struct hb_media_fns bcastOps ={
 	bcast_new,	/* Create single object function */
 	NULL,		/* whole-line parse function */
@@ -117,7 +91,7 @@ static struct hb_media_fns bcastOps ={
 	bcast_isping,
 };
 
-PIL_PLUGIN_BOILERPLATE("1.0", Debug, bcastclosepi);
+PIL_PLUGIN_BOILERPLATE2("1.0", Debug)
 static const PILPluginImports*  PluginImports;
 static PILPlugin*               OurPlugin;
 static PILInterface*		OurInterface;
@@ -148,7 +122,7 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
  	return imports->register_interface(us, PIL_PLUGINTYPE_S
 	,	PIL_PLUGIN_S
 	,	&bcastOps
-	,	bcastcloseintf		/*close */
+	,	NULL		/*close */
 	,	&OurInterface
 	,	(void*)&OurImports
 	,	interfprivate); 
@@ -164,17 +138,18 @@ bcast_mtype(char** buffer) {
 		return 0;
 	}
 
-	return STRLEN(PIL_PLUGIN_S);
+	return STRLEN_CONST(PIL_PLUGIN_S);
 }
 
 static int
 bcast_descr(char **buffer) { 
-	*buffer = STRDUP("UDP/IP broadcast");
+	const char constret[] = "UDP/IP broadcast";
+	*buffer = STRDUP(constret);
 	if (!*buffer) {
 		return 0;
 	}
 
-	return strlen(*buffer);
+	return STRLEN_CONST(constret);
 }
 
 static int
@@ -192,8 +167,8 @@ bcast_init(void)
 	if (localudpport <= 0) {
 		const char *	chport;
 		if ((chport  = OurImports->ParamValue("udpport")) != NULL) {
-			sscanf(chport, "%d", &localudpport);
-			if (localudpport <= 0) {
+			if (sscanf(chport, "%d", &localudpport) <= 0
+				|| localudpport <= 0) {
 				PILCallLog(LOG, PIL_CRIT
 				,	"bad port number %s"
 				,	chport);
@@ -314,11 +289,13 @@ bcast_close(struct hb_media* mp)
 		if (close(ei->rsocket) < 0) {
 			rc = HA_FAIL;
 		}
+		ei->rsocket=-1;
 	}
 	if (ei->wsocket >= 0) {
 		if (close(ei->wsocket) < 0) {
 			rc = HA_FAIL;
 		}
+		ei->wsocket=-1;
 	}
 	PILCallLog(LOG, PIL_INFO
 	, "UDP Broadcast heartbeat closed on port %d interface %s - Status: %d"
@@ -334,15 +311,14 @@ bcast_close(struct hb_media* mp)
  * Receive a heartbeat broadcast packet from BCAST interface
  */
 
+char			bcast_pkt[MAXMSG];
 void *
 bcast_read(struct hb_media* mp, int * lenp)
 {
 	struct ip_private *	ei;
-	char			buf[MAXLINE];
-	int			addr_len = sizeof(struct sockaddr);
+	socklen_t		addr_len = sizeof(struct sockaddr);
    	struct sockaddr_in	their_addr; /* connector's addr information */
 	int	numbytes;
-	void	*pkt;
 
 	BCASTASSERT(mp);
 	ei = (struct ip_private *) mp->pd;
@@ -353,7 +329,7 @@ bcast_read(struct hb_media* mp, int * lenp)
 			   ,	ei->rsocket, ei->wsocket);
 	}
 
-	if ((numbytes=recvfrom(ei->rsocket, buf, MAXLINE-1, MSG_WAITALL
+	if ((numbytes=recvfrom(ei->rsocket, bcast_pkt, MAXMSG-1, MSG_WAITALL
 	,	(struct sockaddr *)&their_addr, &addr_len)) == -1) {
 		if (errno != EINTR) {
 			PILCallLog(LOG, PIL_CRIT
@@ -363,27 +339,19 @@ bcast_read(struct hb_media* mp, int * lenp)
 		return NULL;
 	}
 	/* Avoid possible buffer overruns */
-	buf[numbytes] = EOS;
+	bcast_pkt[numbytes] = EOS;
 
 	if (DEBUGPKT) {
 		PILCallLog(LOG, PIL_DEBUG, "got %d byte packet from %s"
 		,	numbytes, inet_ntoa(their_addr.sin_addr));
 	}
 	if (DEBUGPKTCONT && numbytes > 0) {
-	  PILCallLog(LOG, PIL_DEBUG, "%s", buf);
+		PILCallLog(LOG, PIL_DEBUG, "%s", bcast_pkt);
 	}
 	
-	pkt = ha_malloc(numbytes + 1);
-	if(!pkt){
-		PILCallLog(LOG, PIL_CRIT
-			    ,	"Error in allocating memory");
-		return(NULL);
-	}
-	
-	memcpy(pkt, buf, numbytes + 1);
 	*lenp = numbytes +1;
 	
-	return(pkt);
+	return bcast_pkt;
 }
 
 
@@ -404,8 +372,24 @@ bcast_write(struct hb_media* mp, void *pkt, int len)
 	if ((rc=sendto(ei->wsocket, pkt, len, 0
 	,	(struct sockaddr *)&ei->addr
 	,	sizeof(struct sockaddr))) != len) {
-		PILCallLog(LOG, PIL_CRIT, "Unable to send bcast [%d] packet: %s"
-		,	rc, strerror(errno));
+
+		struct ha_msg* m;
+
+		int		err = errno;
+		if (!mp->suppresserrs) {
+			PILCallLog(LOG, PIL_CRIT, "Unable to send bcast [%d] packet(len=%d): %s"
+			,	rc,len, strerror(err));
+		}
+		
+		if (ANYDEBUG) {
+			m =  wirefmt2msg(pkt, len,MSG_NEEDAUTH);
+			if (m){
+				cl_log_message(LOG_ERR, m);
+				ha_msg_del(m);
+			}
+		}
+
+		errno = err;
 		return(HA_FAIL);
 	}
 
@@ -658,6 +642,7 @@ new_ip_interface(const char * ifn, int port)
 	if (ep == NULL)  {
 		return(NULL);
 	}
+	memset(ep, 0, sizeof(*ep));
 
 	ep->bcast = broadaddr;
 
@@ -781,193 +766,3 @@ if_get_broadaddr(const char *ifn, struct in_addr *broadaddr)
 	
 } /* end if_get_broadaddr() */
 
-/*
- * $Log: bcast.c,v $
- * Revision 1.33.2.3  2004/09/11 06:36:33  msoffen
- * Moved variable  definitions to top of functions/blocks, changed comments to c style comments.
- *
- * Revision 1.37  2004/04/29 16:22:36  msoffen
- * Changed ??? to ? ( picked up as trigraph )
- *
- * Revision 1.36  2004/04/28 22:30:29  alan
- * Put in some fixes for extra freeing of memory in the communications plugins.
- *
- * Revision 1.35  2004/03/03 05:31:50  alan
- * Put in Gochun Shi's new netstrings on-the-wire data format code.
- * this allows sending binary data, among many other things!
- *
- * Revision 1.34  2004/02/17 22:11:59  lars
- * Pet peeve removal: _Id et al now gone, replaced with consistent Id header.
- *
- * Revision 1.33  2004/01/21 11:34:15  horms
- * - Replaced numerous malloc + strcpy/strncpy invocations with strdup
- *   * This usually makes the code a bit cleaner
- *   * Also is easier not to make code with potential buffer over-runs
- * - Added STRDUP to pils modules
- * - Removed some spurious MALLOC and FREE redefinitions
- *   _that could never be used_
- * - Make sure the return value of strdup is honoured in error conditions
- *
- * Revision 1.32  2003/06/24 06:50:13  alan
- * Added a missing format string.
- * With the right level of debugging enabled, this could have allowed
- * a hole to occur.
- *
- * Revision 1.31  2003/02/07 08:37:17  horms
- * Removed inclusion of portability.h from .h files
- * so that it does not need to be installed.
- *
- * Revision 1.30  2003/02/05 09:06:34  horms
- * Lars put a lot of work into making sure that portability.h
- * is included first, everywhere. However this broke a few
- * things when building against heartbeat headers that
- * have been installed (usually somewhere under /usr/include or
- * /usr/local/include).
- *
- * This patch should resolve this problem without undoing all of
- * Lars's hard work.
- *
- * As an asside: I think that portability.h is a virus that has
- * infected all of heartbeat's code and now must also infect all
- * code that builds against heartbeat. I wish that it didn't need
- * to be included all over the place. Especially in headers to
- * be installed on the system. However, I respect Lars's opinion
- * that this is the best way to resolve some weird build problems
- * in the current tree.
- *
- * Revision 1.29  2003/01/31 10:02:09  lars
- * Various small code cleanups:
- * - Lots of "signed vs unsigned" comparison fixes
- * - time_t globally replaced with TIME_T
- * - All seqnos moved to "seqno_t", which defaults to unsigned long
- * - DIMOF() definition centralized to portability.h and typecast to int
- * - EOS define moved to portability.h
- * - dropped inclusion of signal.h from stonith.h, so that sigignore is
- *   properly defined
- *
- * Revision 1.28  2003/01/08 21:14:08  msoffen
- * Fixed problems that prevented compiling with -Wtraditional
- *
- * Revision 1.27  2002/10/22 17:41:58  alan
- * Added some documentation about deadtime, etc.
- * Switched one of the sets of FIFOs to IPC channels.
- * Added msg_from_IPC to ha_msg.c make that easier.
- * Fixed a few compile errors that were introduced earlier.
- * Moved hb_api_core.h out of the global include directory,
- * and back into a local directory.  I also make sure it doesn't get
- * installed.  This *shouldn't* cause problems.
- * Added a ipc_waitin() function to the IPC code to allow you to wait for
- * input synchronously if you really want to.
- * Changes the STONITH test to default to enabled.
- *
- * Revision 1.26  2002/10/02 13:40:26  alan
- * Fixed some potential holes pointed out by Nathan Wallwork.
- *
- * Revision 1.25  2002/09/19 22:40:17  alan
- * Changed a few error return checks to not print anything and return
- * if an error was encountered.
- * Changed a few debug messages to only print if a strictly positive number
- * of chars was received.
- *
- * Revision 1.24  2002/09/12 03:52:07  alan
- * Fixed up a comment :-(.
- *
- * Revision 1.23  2002/09/12 03:39:45  alan
- * Fixed some logging level names in the code and also
- * fixed an error in the license chosen for a file.
- *
- * Revision 1.22  2002/09/11 07:11:23  alan
- * Updated a good bit of code to use logging functions more uniformly.
- * Some used printfs, and some used ha_log, and some used the PILS
- * function pointers (which sidesteps the printf argument checking).
- *
- * Revision 1.21  2002/08/07 18:20:33  msoffen
- * Cleaned up many warning messages from FreeBSD and Solaris.
- * Cleaned up signal calls with SIG_IGN for Solaris to use sigignore function (to remove some warnings).
- *
- * Revision 1.20  2002/07/16 17:10:32  msoffen
- * Changed so that Lars shouldn't have the issue with the struct copy.
- *
- * Revision 1.19  2002/07/16 16:18:15  msoffen
- * -Re-added in LArs's comments.
- *
- * Revision 1.18  2002/07/16 16:16:06  msoffen
- * Changed to NOT assume that the address is 1st.
- *
- * Revision 1.17  2002/07/16 11:47:53 lars
- * Type and alignment fixes for IA64, x86_64, sparc, s390 and PPC(64).
- *
- * Revision 1.16  2002/06/16 06:11:26  alan
- * Put in a couple of changes to the PILS interfaces
- *  - exported license information (name, URL)
- *  - imported malloc/free
- *
- * Revision 1.15  2002/06/16 03:42:53  alan
- * Incorporated code from Stphane Billiart to add a unicast heartbeat capability
- * to heartbeat.
- * Fixed a small bug in bcast.c where the "old" logging function was called
- * instead of the new one.
- *
- * Revision 1.14  2002/05/01 23:50:35  alan
- * Put in some comments about how the code avoids potential buffer overruns.
- *
- * Revision 1.13  2002/04/13 22:35:08  alan
- * Changed ha_msg_add_nv to take an end pointer to make it safer.
- * Added a length parameter to string2msg so it would be safer.
- * Changed the various networking plugins to use the new string2msg().
- *
- * Revision 1.12  2002/04/09 12:45:36  alan
- * Put in changes to the bcast, mcast and serial code such that
- * interrupted system calls in reads are ignored.
- *
- * Revision 1.11  2002/03/25 23:54:52  alan
- * Put in fixes to two bugs noted by Gamid Isayev of netzilla networks.
- * One of the two fixes is also due to him.
- *
- * Revision 1.10  2002/03/25 23:19:36  horms
- * Patches from Gamid Isayev to
- *
- * 1) Avoid shadowing the udpport global, hence this
- *    configuration option should now work.
- *
- * 2) Call getservbyname with a vaild protocol (udp), this should
- *    also allow the udpport configuration option to work.
- *
- * --
- * Horms
- *
- * Revision 1.9  2001/10/03 05:33:28  alan
- * Added a little more error checking for the broadcast port number...
- *
- * Revision 1.8  2001/10/02 20:15:41  alan
- * Debug code, etc. from Matt Soffen...
- *
- * Revision 1.7  2001/10/02 15:52:20  alan
- * Replaced bzero with memset...  This is a minor portability issue...
- *
- * Revision 1.6  2001/10/02 05:12:19  alan
- * Various portability fixes (make warnings go away) for Solaris.
- *
- * Revision 1.5  2001/09/07 16:18:17  alan
- * Updated ping.c to conform to the new plugin loading system.
- * Changed log messages in bcast, mcast, ping and serial to use the
- * new logging function.
- *
- * Revision 1.4  2001/08/15 16:17:12  alan
- * Fixed the code so that serial comm plugins build/load/work.
- *
- * Revision 1.3  2001/08/15 02:14:49  alan
- * Put in #include net/if.h needed for FreeBSD and solaris
- *
- * Revision 1.2  2001/08/11 01:40:54  alan
- * Removed the old copy of the heartbeat.h file, and removed a
- * blank line from a file... ;-)
- *
- * Revision 1.1  2001/08/10 17:16:44  alan
- * New code for the new plugin loading system.
- *
- * Revision 1.20  2001/06/19 13:56:28  alan
- * FreeBSD portability patch from Matt Soffen.
- * Mainly added #include "portability.h" to lots of files.
- * Also added a library to Makefile.am
- */
