@@ -34,8 +34,8 @@
  *
  */
 
+#include <lha_internal.h>
 #include <time.h>
-#include <portability.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -110,31 +110,6 @@ static int		hbaping_isping(void);
 #define		ISPINGOBJECT(mp)	((mp) && ((mp)->vf == (void*)&hbapingOps))
 #define		PINGASSERT(mp)	g_assert(ISPINGOBJECT(mp))
 
-/*
- * pingclose is called as part of unloading the ping HBcomm plugin.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the plugin, and not a single interface
- * in particular, here's our chance to clean it up.
- */
-
-static void
-hbapingclosepi(PILPlugin*pi)
-{
-}
-
-
-/*
- * pingcloseintf called as part of shutting down the ping HBcomm interface.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the ping implementation, here's our chance
- * to clean it up.
- */
-static PIL_rc
-hbapingcloseintf(PILInterface* pi, void* pd)
-{
-	return PIL_OK;
-}
-
 static struct hb_media_fns hbapingOps ={
 	hbaping_new,	/* Create single object function */
 	NULL,		/* whole-line parse function */
@@ -147,7 +122,7 @@ static struct hb_media_fns hbapingOps ={
 	hbaping_isping,
 };
 
-PIL_PLUGIN_BOILERPLATE("1.0", Debug, hbapingclosepi);
+PIL_PLUGIN_BOILERPLATE2("1.0", Debug);
 
 static const PILPluginImports*  PluginImports;
 static PILPlugin*               OurPlugin;
@@ -179,7 +154,7 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
  	return imports->register_interface(us, PIL_PLUGINTYPE_S
 	,	PIL_PLUGIN_S
 	,	&hbapingOps
-	,	hbapingcloseintf		/*close */
+	,	NULL		/*close */
 	,	&OurInterface
 	,	(void*)&OurImports
 	,	interfprivate); 
@@ -262,6 +237,7 @@ hbaping_new(const char * host)
 
 	ret = (struct hb_media *) MALLOC(sizeof(struct hb_media));
 	if (ret != NULL) {
+		memset(ret, 0, sizeof(*ret));
 		char * name;
 		ret->pd = (void*)ipi;
 		name = MALLOC(strlen(host)+1);
@@ -276,7 +252,7 @@ hbaping_new(const char * host)
 }
 
 /*
- *	Close UDP/IP broadcast heartbeat interface
+ *	Close HBA connection
  */
 
 static int
@@ -288,7 +264,10 @@ hbaping_close(struct hb_media* mp)
 	PINGASSERT(mp);
 	ei = (struct hbaping_private *) mp->pd;
 
-	HBA_CloseAdapter(ei->handle);
+	if (ei->handle >= 0) {
+		HBA_CloseAdapter(ei->handle);
+		ei->handle=-1;
+	}
 
 	return(rc);
 }
@@ -299,6 +278,7 @@ hbaping_close(struct hb_media* mp)
  * successful.
  */
 
+char hbaping_pkt[MAXLINE];
 static void *
 hbaping_read(struct hb_media* mp, int *lenp)
 {
@@ -351,7 +331,10 @@ hbaping_read(struct hb_media* mp, int *lenp)
 	
 	ha_msg_del(nmsg);
 	
-	return pkt;
+	memcpy(hbaping_pkt, pkt, *lenp);
+	cl_free(pkt);
+	
+	return hbaping_pkt;
 	
 }
 
