@@ -1,4 +1,3 @@
-/* $Id: ping_group.c,v 1.4.2.5 2005/08/10 04:08:17 horms Exp $ */
 /*
  * ping_group.c: ICMP-echo-based heartbeat code for heartbeat.
  *
@@ -35,9 +34,10 @@
  *
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
@@ -79,6 +79,7 @@
 #include <heartbeat.h>
 #include <HBcomm.h>
 #include <clplumbing/realtime.h>
+#include <clplumbing/uids.h>
 
 #ifdef linux
 #	define	ICMP_HDR_SZ	sizeof(struct icmphdr)	/* 8 */
@@ -135,31 +136,6 @@ static int		ping_group_isping(void);
 			((mp) && ((mp)->vf == (void*)&ping_group_ops))
 #define		PINGGROUPASSERT(mp)	g_assert(ISPINGGROUPOBJECT(mp))
 
-/*
- * pingclose is called as part of unloading the ping HBcomm plugin.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the plugin, and not a single interface
- * in particular, here's our chance to clean it up.
- */
-
-static void
-ping_group_close_pi(PILPlugin*pi)
-{
-}
-
-
-/*
- * ping_group_close_intf called as part of shutting down the ping HBcomm interface.
- * If there was any global data allocated, or file descriptors opened, etc.
- * which is associated with the ping implementation, here's our chance
- * to clean it up.
- */
-static PIL_rc
-ping_group_close_intf(PILInterface* pi, void* pd)
-{
-	return PIL_OK;
-}
-
 static struct hb_media_fns ping_group_ops ={
 	NULL,		  /* Create single object function */
 	ping_group_parse, /* whole-line parse function */
@@ -172,7 +148,7 @@ static struct hb_media_fns ping_group_ops ={
 	ping_group_isping,
 };
 
-PIL_PLUGIN_BOILERPLATE("1.0", Debug, ping_group_close_pi);
+PIL_PLUGIN_BOILERPLATE2("1.0", Debug)
 
 static const PILPluginImports*  PluginImports;
 static PILPlugin*               OurPlugin;
@@ -204,7 +180,7 @@ PIL_PLUGIN_INIT(PILPlugin*us, const PILPluginImports* imports)
  	return imports->register_interface(us, PIL_PLUGINTYPE_S
 	,	PIL_PLUGIN_S
 	,	&ping_group_ops
-	,	ping_group_close_intf		/*close */
+	,	NULL		/*close */
 	,	&OurInterface
 	,	(void*)&OurImports
 	,	interfprivate); 
@@ -311,10 +287,11 @@ ping_group_new(const char *name)
 	priv->ident = getpid() & 0xFFFF;
 
 	media = (struct hb_media *) MALLOC(sizeof(struct hb_media));
-	if(!media) {
+	if (!media) {
 		FREE(priv);
 		return(NULL);
 	}
+	memset(media, 0, sizeof(*media));
 
 	media->pd = (void*)priv;
 	tmp = STRDUP(name);
@@ -389,6 +366,7 @@ ping_group_close(struct hb_media* mp)
 		if (close(ei->sock) < 0) {
 			rc = HA_FAIL;
 		}
+		ei->sock=-1;
 	}
 
 	ping_group_destroy_data(mp);
@@ -401,6 +379,7 @@ ping_group_close(struct hb_media* mp)
  * Receive a heartbeat ping reply packet.
  */
 
+char	   pinggroup_pkt[MAXLINE];
 static void *
 ping_group_read(struct hb_media* mp, int *lenp)
 {
@@ -423,14 +402,14 @@ ping_group_read(struct hb_media* mp, int *lenp)
 	ping_group_node_t	*node;
 	struct ha_msg		*msg = NULL;
 	const char 		*comment;
-	char			*pkt;
 	int			pktlen;
 	
 	PINGGROUPASSERT(mp);
 	ei = (ping_group_private_t *) mp->pd;
 
-ReRead:        /* We recv lots of packets that aren't ours */
+ReRead:	/* We recv lots of packets that aren't ours */
 
+	*lenp = 0;
 	if ((numbytes=recvfrom(ei->sock, (void *) &buf.cbuf
 	,	sizeof(buf.cbuf)-1, 0,	(struct sockaddr *)&their_addr
 	,	&addr_len)) < 0) {
@@ -473,18 +452,17 @@ ReRead:        /* We recv lots of packets that aren't ours */
 		PILCallLog(LOG, PIL_DEBUG, "%s", msgstart);
 	}
 
-
 	for(node = ei->node; node; node = node->next) {
-		if(!memcmp(&(their_addr.sin_addr), &(node->addr.sin_addr),
-					sizeof(struct in_addr))) {
-			goto ReRead;    /* Not ours */
+		if(!memcmp(&(their_addr.sin_addr), &(node->addr.sin_addr)
+		,	sizeof(struct in_addr))) {
+			break;
 		}
 	}
 	if(!node) {
-		return(NULL);
+		goto ReRead;	/* Not ours */
 	}
 
-	msg = wirefmt2msg(msgstart, bufmax - msgstart);
+	msg = wirefmt2msg(msgstart, bufmax - msgstart, MSG_NEEDAUTH);
 	if(msg == NULL) {
 		errno = EINVAL;
 		return(NULL);
@@ -506,18 +484,13 @@ ReRead:        /* We recv lots of packets that aren't ours */
 	ei->slot[slotn] = seq;
 	
 	pktlen = numbytes - hlen - ICMP_HDR_SZ;
-	if (NULL == (pkt = ha_malloc(pktlen + 1))) {
-		ha_msg_del(msg);
-		errno = ENOMEM;
-		return NULL;
-	}
-	pkt[pktlen] = 0;
+	pinggroup_pkt[pktlen] = 0;
 	
-	memcpy(pkt, buf.cbuf + hlen + ICMP_HDR_SZ, pktlen);	
+	memcpy(pinggroup_pkt, buf.cbuf + hlen + ICMP_HDR_SZ, pktlen);	
 	*lenp = pktlen + 1;
 	
 	ha_msg_del(msg);
-	return(pkt);
+	return(pinggroup_pkt);
 }
 
 /*
@@ -551,11 +524,12 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 	struct ha_msg *		nmsg;
 	ping_group_node_t *	node;
 	struct ha_msg*		msg;
-
-	PINGGROUPASSERT(mp);
-
+	static gboolean		needroot=FALSE;			
 	
-	if ((msg = wirefmt2msg(p, len)) == NULL) {
+	PINGGROUPASSERT(mp);
+	
+	
+	if ((msg = wirefmt2msg(p, len, MSG_NEEDAUTH)) == NULL) {
 		PILCallLog(LOG, PIL_CRIT, "ping_write(): cannot convert wirefmt to msg");
 		return(HA_FAIL);
 	}
@@ -583,6 +557,7 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 	 */
 	if ((nmsg = ha_msg_new(5)) == NULL) {
 		PILCallLog(LOG, PIL_CRIT, "cannot create new message");
+		ha_msg_del(msg);
 		return(HA_FAIL);
 	}
 
@@ -616,7 +591,7 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 
 	if ((icmp_pkt = MALLOC(pktsize)) == NULL) {
 		PILCallLog(LOG, PIL_CRIT, "out of memory");
-		ha_free(pkt);
+		cl_free(pkt);
 		ha_msg_del(msg);
 		return HA_FAIL;
 	}
@@ -630,17 +605,32 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 	++ei->iseq;
 
 	memcpy(icp->icmp_data, pkt, size);
-	ha_free(pkt); pkt = NULL;
+	cl_free(pkt); pkt = NULL;
 
 	/* Compute the ICMP checksum */
 	icp->icmp_cksum = in_cksum((u_short *)icp, pktsize);
 
+retry:
+	if (needroot) {
+		return_to_orig_privs();
+	}
+
 	for(node = ei->node; node; node = node->next) {
-		if ((rc=sendto(ei->sock, (void *) icmp_pkt, pktsize, 0
+		if ((rc=sendto(ei->sock, (void *) icmp_pkt, pktsize
+		,	MSG_DONTWAIT
 		,	(struct sockaddr *)&node->addr
 		,	sizeof(struct sockaddr))) != (ssize_t)pktsize) {
-			PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s"
-			,	strerror(errno));
+			if (errno == EPERM && !needroot) {
+				needroot=TRUE;
+				goto retry;
+			}
+			if (!mp->suppresserrs) {
+				PILCallLog(LOG, PIL_CRIT, "Error sending packet: %s"
+				,	strerror(errno));
+				PILCallLog(LOG, PIL_INFO, "euid=%lu egid=%lu"
+				,	(unsigned long) geteuid()
+				,	(unsigned long) getegid());
+			}
 			FREE(icmp_pkt);
 			ha_msg_del(msg);
 			return(HA_FAIL);
@@ -653,9 +643,13 @@ ping_group_write(struct hb_media* mp, void *p, int len)
 
 		cl_shortsleep();
 	}
+	if (needroot) {
+		return_to_dropped_privs();
+	}
 
 	if (DEBUGPKTCONT) {
-		PILCallLog(LOG, PIL_DEBUG, "%s", (const char*)pkt);
+		PILCallLog(LOG, PIL_DEBUG, "%s"
+		,	(const char*)icp->icmp_data);
    	}
 
 	FREE(icmp_pkt);
