@@ -33,9 +33,13 @@
 #include <errno.h>
 #include <glib.h>
 #include <clplumbing/cl_log.h>
+#include <clplumbing/realtime.h>
 #include <pils/plugin.h>
 #include <dirent.h>
 #include <libgen.h>  /* Add it for compiling on OSX */
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 
 #include <lrm/raexec.h>
 
@@ -297,6 +301,7 @@ get_resource_meta(const char* rsc_type, const char* provider)
 	char ra_pathname[RA_MAX_NAME_LENGTH];
 	FILE* file = NULL;
 	GHashTable * tmp_for_setenv;
+	struct timespec short_sleep = {0,200000000L}; /*20ms*/
 
 	get_ra_pathname(RA_PATH, rsc_type, provider, ra_pathname);
 
@@ -314,13 +319,13 @@ get_resource_meta(const char* rsc_type, const char* provider)
 
 	g_str_tmp = g_string_new("");
 	while(!feof(file)) {
-		memset(buff, 0, BUFF_LEN);
 		read_len = fread(buff, 1, BUFF_LEN - 1, file);
 		if (0<read_len) {
+			*(buff+read_len) = '\0';
 			g_string_append(g_str_tmp, buff);
 		}
 		else {
-			sleep(1);
+			nanosleep(&short_sleep,NULL);
 		}
 	}
 	if (0 == g_str_tmp->len) {
