@@ -1,4 +1,3 @@
-/* $Id: ipmilan.h,v 1.1.2.3 2004/12/01 00:35:01 yixiong Exp $ */
 /*
  * Stonith module for ipmi lan Stonith device
  *
@@ -30,8 +29,8 @@ struct ipmilanHostInfo {
 	int 		portnumber;
 	int 		authtype;
 	int		privilege;
-	char 		username[17];
-	char 		password[17];
+	char * 		username;
+	char *		password;
 
 	struct ipmilanHostInfo *  prev;
 	struct ipmilanHostInfo *  next;
