@@ -1,27 +1,25 @@
-/* $Id: ccm_testclient.c,v 1.13.4.1 2004/04/20 05:05:34 alan Exp $ */
 /* 
  * ccm.c: A consensus cluster membership sample client
  *
  * Copyright (c) International Business Machines  Corp., 2000
  * Author: Ram Pai (linuxram@us.ibm.com)
  * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
-#include <portability.h>
+
+#include <lha_internal.h>
 #include <ocf/oc_event.h>
 #include <stdio.h>
 #include <sys/time.h>
@@ -45,28 +43,31 @@ my_ms_events(oc_ed_t event, void *cookie,
 	uint i;
 	int i_am_in;
 
- 	cl_log(LOG_INFO,"event=%s", 
-			event==OC_EV_MS_NEW_MEMBERSHIP?"NEW MEMBERSHIP":
-		        event==OC_EV_MS_NOT_PRIMARY?"NOT PRIMARY":
-			event==OC_EV_MS_PRIMARY_RESTORED?"PRIMARY RESTORED":
-			event==OC_EV_MS_EVICTED?"EVICTED":
-			      "NO QUORUM MEMBERSHIP");
+ 	cl_log(LOG_INFO,"event=%s:", 
+	       event==OC_EV_MS_NEW_MEMBERSHIP?"NEW MEMBERSHIP":
+	       event==OC_EV_MS_NOT_PRIMARY?"NOT PRIMARY":
+	       event==OC_EV_MS_PRIMARY_RESTORED?"PRIMARY RESTORED":
+	       event==OC_EV_MS_EVICTED?"EVICTED":
+	       "NO QUORUM MEMBERSHIP"
+	       );
 
 	if(OC_EV_MS_EVICTED == event) {
 		oc_ev_callback_done(cookie);
 		return;
 	}
 
-	cl_log(LOG_INFO,"trans=%d, nodes=%d, new=%d, lost=%d n_idx=%d, "
-				"new_idx=%d, old_idx=%d",
-			oc->m_instance,
-			oc->m_n_member,
-			oc->m_n_in,
-			oc->m_n_out,
-			oc->m_memb_idx,
-			oc->m_in_idx,
-			oc->m_out_idx);
-
+	cl_log(LOG_INFO,"instance=%d\n"
+	       "# ttl members=%d, ttl_idx=%d\n"
+	       "# new members=%d, new_idx=%d\n"
+	       "# out members=%d, out_idx=%d",
+	       oc->m_instance,
+	       oc->m_n_member,
+	       oc->m_memb_idx,
+	       oc->m_n_in,
+	       oc->m_in_idx,
+	       oc->m_n_out,	       
+	       oc->m_out_idx);
+	
 	i_am_in=0;
 	cl_log(LOG_INFO, "NODES IN THE PRIMARY MEMBERSHIP");
 	for(i=0; i<oc->m_n_member; i++) {
