@@ -19,8 +19,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#include <portability.h>
+#include <lha_internal.h>
 #include <strings.h>
+#include <stdio.h>
 
 #include <unistd.h>
 #include <glib.h>
@@ -140,36 +141,40 @@ saClmInitialize(SaClmHandleT *clmHandle, const SaClmCallbacksT *clmCallbacks,
 	int ret;
 	oc_ev_t *ev_token;
 	__clm_handle_t *hd;
-	__clm_handle_t **phd;
 	SaClmHandleT *hash_key;
 	fd_set rset;
 	struct timeval tv;
+        SaErrorT rc;
 
 	oc_ev_register(&ev_token);
 	if ((ret = oc_ev_set_callback(ev_token, OC_EV_MEMB_CLASS
 	,	ccm_events, NULL)) != 0) {
-		if (ret == ENOMEM)
-			return SA_ERR_NO_MEMORY;
-		else
+		if (ret == ENOMEM){
+			rc = SA_ERR_NO_MEMORY;
+                        goto err_nomem_exit;
+		}
+		else{
 			assert(0);	/* Never runs here */
+		}
 	}
 	/* We must call it to get non-quorum partition info */
 	oc_ev_special(ev_token, OC_EV_MEMB_CLASS, 0);
 
 	clm_init();
 
-	phd = (__clm_handle_t **)g_malloc(sizeof(__clm_handle_t *));
-	if (!phd)
-		return SA_ERR_NO_MEMORY;
 
 	hash_key = (SaClmHandleT *)g_malloc(sizeof(SaClmHandleT));
-	if (!hash_key)
-		return SA_ERR_NO_MEMORY;
+	if (!hash_key){
+		rc = SA_ERR_NO_MEMORY;
+                goto err_nomem_exit;
+	}
 
-	*phd = (__clm_handle_t *)g_malloc(sizeof(__clm_handle_t));
-	hd = *phd;
-	if (!hd)
-		return SA_ERR_NO_MEMORY;
+	hd = (__clm_handle_t *)g_malloc(sizeof(__clm_handle_t));
+	if (!hd){
+                g_free(hash_key);
+		rc = SA_ERR_NO_MEMORY;
+                goto err_nomem_exit;
+	}
 
 	*clmHandle = __handle_counter++;
 	*hash_key = *clmHandle;
@@ -181,32 +186,48 @@ saClmInitialize(SaClmHandleT *clmHandle, const SaClmCallbacksT *clmCallbacks,
 
 	if ((ret = oc_ev_activate(hd->ev_token, &hd->fd)) != 0) {
 		cl_log(LOG_ERR, "oc_ev_activate error [%d]", ret);
-		return SA_ERR_LIBRARY;
+		rc = SA_ERR_LIBRARY;
+                goto err_lib_exit;
 	}
 
 	/* Prepare information for saClmClusterNodeGet() series calls */
-	FD_ZERO(&rset);
-	FD_SET(hd->fd, &rset);
-	tv.tv_sec = 2;
-	tv.tv_usec = 0;
+	while (!__ccm_data) {
 
-	if ((ret = select(hd->fd + 1, &rset, NULL, NULL, &tv)) == -1) {
-		cl_log(LOG_ERR, "%s: select error [%d]"
-		,	__FUNCTION__, ret);
-		return SA_ERR_LIBRARY;
+		FD_ZERO(&rset);
+		FD_SET(hd->fd, &rset);
+		tv.tv_sec = 2;
+		tv.tv_usec = 0;
+
+		if ((ret = select(hd->fd + 1, &rset, NULL, NULL, &tv)) == -1) {
+			cl_log(LOG_ERR, "%s: select error [%d]"
+			,	__FUNCTION__, ret);
+			rc = SA_ERR_LIBRARY;
+                        goto err_lib_exit;
+
+		} else if (ret == 0) {
+			cl_log(LOG_WARNING, "%s: select timeout", __FUNCTION__);
+			rc = SA_ERR_TIMEOUT;
+                        goto err_lib_exit;
+		}
+
+		if ((ret = oc_ev_handle_event(hd->ev_token) != 0)) {
+			cl_log(LOG_ERR, "%s: oc_ev_handle_event error [%d]"
+			,	__FUNCTION__, ret);
+			rc = SA_ERR_LIBRARY;
+                        goto err_lib_exit;
+		}
 
-	} else if (ret == 0) {
-		cl_log(LOG_WARNING, "%s: select timeout", __FUNCTION__);
-		return SA_ERR_TIMEOUT;
 	}
-
-	if ((ret = oc_ev_handle_event(hd->ev_token) != 0)) {
-		cl_log(LOG_ERR, "%s: oc_ev_handle_event error [%d]"
-		,	__FUNCTION__, ret);
-		return SA_ERR_LIBRARY;
-	}
-
 	return SA_OK;
+
+ err_lib_exit:
+        g_hash_table_remove(__handle_hash, hash_key);
+        g_free(hd);
+        g_free(hash_key);
+
+ err_nomem_exit:
+        oc_ev_unregister(ev_token);
+        return rc;
 }
 
 SaErrorT 
@@ -215,8 +236,9 @@ saClmSelectionObjectGet(const SaClmHandleT *clmHandle,
 {
 	__clm_handle_t *hd = GET_CLM_HANDLE(clmHandle);
 
-	if (!hd)
+	if (!hd){
 		return SA_ERR_BAD_HANDLE;
+	}
 
 	*selectionObject = hd->fd;
 
@@ -251,7 +273,7 @@ retrieve_current_buffer(__clm_handle_t *hd)
 		MEMNODE(i).member = 1;
 		p = oc->m_array[oc->m_memb_idx+i].node_uname;
 		if (p) {
-			strncpy((char*)MEMNODE(i).nodeName.value, p, 
+			strncpy((char *)MEMNODE(i).nodeName.value, p, 
 					SA_MAX_NAME_LENGTH - 1);
 			MEMNODE(i).nodeName.value[SA_MAX_NAME_LENGTH-1] = '\0';
 		} else {
@@ -334,7 +356,7 @@ retrieve_changes_only_buffer(__clm_handle_t *hd)
 		MEMNODE(n).member = 0;
 		p = oc->m_array[oc->m_out_idx+i].node_uname;
 		if (p) {
-			strncpy((char*)MEMNODE(n).nodeName.value, p,
+			strncpy((char *)MEMNODE(n).nodeName.value, p,
 					SA_MAX_NAME_LENGTH - 1);
 			MEMNODE(n).nodeName.value[SA_MAX_NAME_LENGTH-1] = '\0';
 		} else {
@@ -353,12 +375,14 @@ saClmDispatch(const SaClmHandleT *clmHandle,
 	uint itemnum;
 	__clm_handle_t *hd = GET_CLM_HANDLE(clmHandle);
 
-	if (!hd)
+	if (!hd){
 		return SA_ERR_BAD_HANDLE;
+	}
 
 	if ((ret = oc_ev_handle_event(hd->ev_token)) != 0) {
-		if (ret == EINVAL)
+		if (ret == EINVAL){
 			return SA_ERR_BAD_HANDLE;
+		}
 
 		/* else we must be evicted */
 	}
@@ -378,13 +402,15 @@ saClmDispatch(const SaClmHandleT *clmHandle,
 		,	__ccm_event);
 		return SA_OK;
 	}
-	if (!__ccm_data)
+	if (!__ccm_data){
 		return SA_ERR_INIT;
+	}
 
 	oc = __ccm_data;
 
-	if(CLM_TRACK_STOP == hd->trackflags)
+	if(CLM_TRACK_STOP == hd->trackflags){
 		return SA_OK;
+	}
 
 	/* SA_TRACK_CURRENT is cleared in saClmClusterTrackStart, hence we 
 	 * needn't to deal with it now*/
@@ -450,8 +476,9 @@ saClmClusterTrackStart(const SaClmHandleT *clmHandle,
 {
 	__clm_handle_t *hd = GET_CLM_HANDLE(clmHandle);
 
-	if (!hd)
+	if (!hd){
 		return SA_ERR_BAD_HANDLE;
+	}
 
 	hd->trackflags = trackFlags;
 	hd->itemnum = numberOfItems;
@@ -463,6 +490,10 @@ saClmClusterTrackStart(const SaClmHandleT *clmHandle,
 		
 		/* Clear SA_TRACK_CURRENT, it's no use since now. */
 		hd->trackflags &= ~SA_TRACK_CURRENT;
+
+		if (__ccm_data == NULL) {
+			return SA_ERR_LIBRARY;
+		}
 		
 		oc = __ccm_data;
 		itemnum = oc->m_n_member;
@@ -488,8 +519,9 @@ saClmClusterTrackStop(const SaClmHandleT *clmHandle)
 {
 	__clm_handle_t *hd = GET_CLM_HANDLE(clmHandle);
 
-	if (!hd)
+	if (!hd){
 		return SA_ERR_BAD_HANDLE;
+	}
 
 	/* This is ugly. But we currently depends on OCF interface, we have
 	 * no choice. This should be fixed in the next version after we remove
@@ -515,7 +547,7 @@ retrieve_node_buffer(SaClmNodeIdT nodeId, SaClmClusterNodeT *clusterNode)
 			clusterNode->member = 1;
 			p = oc->m_array[oc->m_memb_idx+i].node_uname;
 			if (p) {
-				strncpy((char*)clusterNode->nodeName.value, p,
+				strncpy((char *)clusterNode->nodeName.value, p,
 						SA_MAX_NAME_LENGTH - 1);
 				clusterNode->nodeName.value \
 					[SA_MAX_NAME_LENGTH-1] = '\0';
@@ -531,7 +563,7 @@ retrieve_node_buffer(SaClmNodeIdT nodeId, SaClmClusterNodeT *clusterNode)
 			clusterNode->member = 0;
 			p = oc->m_array[oc->m_out_idx+i].node_uname;
 			if (p) {
-				strncpy((char*)clusterNode->nodeName.value, p,
+				strncpy((char *)clusterNode->nodeName.value, p,
 						SA_MAX_NAME_LENGTH - 1);
 				clusterNode->nodeName.value \
 					[SA_MAX_NAME_LENGTH-1] = '\0';
@@ -564,12 +596,14 @@ saClmClusterNodeGet(SaClmNodeIdT nodeId, SaTimeT timeout,
 		return SA_ERR_INVALID_PARAM;
 	}
 	for (i = 0; i < timeout; i++) {
-		if (__ccm_data)
+		if (__ccm_data){
 			break;
+		}
 		sleep(1);
 	}
-	if (i == timeout)
+	if (i == timeout){
 		return SA_ERR_TIMEOUT;
+	}
 
 	pthread_lock();
 	ret = retrieve_node_buffer(nodeId, clusterNode);
@@ -590,8 +624,9 @@ saClmClusterNodeGetAsync(const SaClmHandleT *clmHandle,
 	int ret;
 	__clm_handle_t *hd = GET_CLM_HANDLE(clmHandle);
 
-	if (!hd)
+	if (!hd){
 		return SA_ERR_BAD_HANDLE;
+	}
 
 	if (!clusterNode) {
 		cl_log(LOG_ERR, "Invalid parameter clusterNode <%p>"
