@@ -49,6 +49,7 @@ static void on_cib_diff(const char *event, HA_Message *msg);
 
 static char* on_get_cib_version(char* argv[], int argc);
 
+static char* on_get_crm_metadata(char* argv[], int argc);
 static char* on_get_crm_config(char* argv[], int argc);
 static char* on_update_crm_config(char* argv[], int argc);
 static char* on_get_activenodes(char* argv[], int argc);
@@ -79,6 +80,10 @@ static char* on_update_rsc_attr(char* argv[], int argc);
 static char* on_get_rsc_running_on(char* argv[], int argc);
 static char* on_get_rsc_status(char* argv[], int argc);
 
+static char* on_get_rsc_metaattrs(char* argv[], int argc);
+static char* on_update_rsc_metaattrs(char* argv[], int argc);
+static char* on_delete_rsc_metaattr(char* argv[], int argc);
+
 static char* on_get_rsc_params(char* argv[], int argc);
 static char* on_update_rsc_params(char* argv[], int argc);
 static char* on_delete_rsc_param(char* argv[], int argc);
@@ -95,8 +100,9 @@ static char* on_get_constraint(char* argv[], int argc);
 static char* on_update_constraint(char* argv[], int argc);
 static char* on_delete_constraint(char* argv[], int argc);
 
+static void get_meta_attributes_id(const char* rsc_id, char* id);
 static void get_instance_attributes_id(const char* rsc_id, char* id);
-static void get_attr_id(const char* rsc_id, const char* attr, char* id);
+static void get_attr_id(const char* rsc_id, const char* attr_type, const char* attr, char* id);
 static int delete_object(const char* type, const char* entry, const char* id, crm_data_t** output);
 static GList* find_xml_node_list(crm_data_t *root, const char *search_path);
 static int refresh_lrm(IPC_Channel *crmd_channel, const char *host_uname);
@@ -263,7 +269,7 @@ get_parent(resource_t* child)
 	cur = data_set->resources;
 	while (cur != NULL) {
 		resource_t* rsc = (resource_t*)cur->data;
-		if(rsc->orphan == FALSE || rsc->role != RSC_ROLE_STOPPED) {
+		if(is_not_set(rsc->flags, pe_rsc_orphan) || rsc->role != RSC_ROLE_STOPPED) {
 			GList* child_list = rsc->fns->children(rsc);
 			if (g_list_find(child_list, child) != NULL) {
 				free_data_set(data_set);
@@ -331,6 +337,39 @@ get_fix(const char* rsc_id, char* prefix, char* suffix, char* real_id)
 	free_data_set(data_set);
 	return 0;
 }
+
+static void
+get_meta_attributes_id(const char* rsc_id, char* id)
+{
+	resource_t* rsc;
+	const char* cur_id;
+	pe_working_set_t* data_set;
+	struct ha_msg* attrs;
+	
+	data_set = get_data_set();
+	rsc = pe_find_resource(data_set->resources, rsc_id);	
+	if (rsc == NULL) {
+		snprintf(id, MAX_STRLEN, "%s_meta_attrs", rsc_id);
+		free_data_set(data_set);
+		return;
+	}
+	attrs = cl_get_struct((struct ha_msg*)rsc->xml, "meta_attributes");
+	if (attrs == NULL) {
+		snprintf(id, MAX_STRLEN, "%s_meta_attrs", rsc_id);
+		free_data_set(data_set);
+		return;
+	}
+	cur_id = ha_msg_value(attrs, "id");
+	if (cur_id == NULL) {
+		snprintf(id, MAX_STRLEN, "%s_meta_attrs", rsc_id);
+		free_data_set(data_set);
+		return;
+	}
+	STRNCPY(id, cur_id, MAX_STRLEN);
+	free_data_set(data_set);
+	return;					
+
+}
 static void
 get_instance_attributes_id(const char* rsc_id, char* id)
 {
@@ -364,7 +403,7 @@ get_instance_attributes_id(const char* rsc_id, char* id)
 
 }
 static void
-get_attr_id(const char* rsc_id, const char* attr, char* id)
+get_attr_id(const char* rsc_id, const char* attr_type, const char* attr, char* id)
 {
 	int i;
 	resource_t* rsc;
@@ -373,24 +412,30 @@ get_attr_id(const char* rsc_id, const char* attr, char* id)
 	struct ha_msg* attrs;
 	struct ha_msg* nvpair;
 	pe_working_set_t* data_set;
+	const char * mid = "";
+
+	if (STRNCMP_CONST(attr_type, "meta_attributes") == 0)
+		mid = "metaattr_";
+	if (STRNCMP_CONST(attr_type, "instance_attributes") == 0)
+		mid = "instattr_";
 	
 	data_set = get_data_set();
 	rsc = pe_find_resource(data_set->resources, rsc_id);	
 	if (rsc == NULL) {
-		snprintf(id, MAX_STRLEN,  "%s_%s", rsc_id, attr);
+		snprintf(id, MAX_STRLEN,  "%s_%s%s", rsc_id, mid, attr);
 		free_data_set(data_set);
 		return;
 	}
 
-	attrs = cl_get_struct((struct ha_msg*)rsc->xml, "instance_attributes");
+	attrs = cl_get_struct((struct ha_msg*)rsc->xml, attr_type);
 	if(attrs == NULL) {
-		snprintf(id, MAX_STRLEN,  "%s_%s", rsc_id, attr);
+		snprintf(id, MAX_STRLEN,  "%s_%s%s", rsc_id, mid, attr);
 		free_data_set(data_set);
 		return;
 	}
 	attrs = cl_get_struct(attrs, "attributes");
 	if(attrs == NULL) {
-		snprintf(id, MAX_STRLEN,  "%s_%s", rsc_id, attr);
+		snprintf(id, MAX_STRLEN,  "%s_%s%s", rsc_id, mid, attr);
 		free_data_set(data_set);
 		return;
 	}
@@ -408,7 +453,7 @@ get_attr_id(const char* rsc_id, const char* attr, char* id)
 			}
 		}
 	}
-	snprintf(id, MAX_STRLEN,  "%s_%s", rsc_id, attr);
+	snprintf(id, MAX_STRLEN,  "%s_%s%s", rsc_id, mid, attr);
 	free_data_set(data_set);
 	return;
 }
@@ -448,6 +493,7 @@ init_crm(int cache_cib)
 			, on_cib_connection_destroy);
 
 	reg_msg(MSG_CIB_VERSION, on_get_cib_version);
+	reg_msg(MSG_CRM_METADATA, on_get_crm_metadata);
 	reg_msg(MSG_CRM_CONFIG, on_get_crm_config);
 	reg_msg(MSG_UP_CRM_CONFIG, on_update_crm_config);
 	
@@ -472,6 +518,9 @@ init_crm(int cache_cib)
 	reg_msg(MSG_RSC_TYPE, on_get_rsc_type);
 	reg_msg(MSG_UP_RSC_ATTR, on_update_rsc_attr);
 		
+	reg_msg(MSG_RSC_METAATTRS, on_get_rsc_metaattrs);
+	reg_msg(MSG_UP_RSC_METAATTRS, on_update_rsc_metaattrs);
+	reg_msg(MSG_DEL_RSC_METAATTR, on_delete_rsc_metaattr);
 	
 	reg_msg(MSG_RSC_PARAMS, on_get_rsc_params);
 	reg_msg(MSG_UP_RSC_PARAMS, on_update_rsc_params);
@@ -553,38 +602,76 @@ on_get_cib_version(char* argv[], int argc)
 	free_data_set(data_set);
 	return ret;
 }
+
+static char*
+on_get_crm_metadata(char* argv[], int argc)
+{
+	char cmd[MAX_STRLEN];
+	char buf[MAX_STRLEN];	
+	char* ret = cl_strdup(MSG_OK);
+	FILE *fstream = NULL;
+
+	ARGC_CHECK(2);
+
+	snprintf(cmd, sizeof(cmd), BIN_DIR"/%s metadata", argv[1]);
+	if ((fstream = popen(cmd, "r")) == NULL){
+		mgmt_log(LOG_ERR, "error on popen %s: %s",
+			 cmd, strerror(errno));
+		return cl_strdup(MSG_FAIL);
+	}
+
+	while (!feof(fstream)){
+		memset(buf, 0, sizeof(buf));
+		if (fgets(buf, sizeof(buf), fstream) != NULL){
+			ret = mgmt_msg_append(ret, buf);
+		}
+		else{
+			sleep(1);
+		}
+	}
+
+	if (pclose(fstream) == -1)
+		mgmt_log(LOG_WARNING, "failed to close pipe");
+
+	return ret;
+}
 char* 
 on_get_crm_config(char* argv[], int argc)
 {
-	char buf [255];
+	const char* value = NULL;
 	pe_working_set_t* data_set;
 	char* ret = cl_strdup(MSG_OK);
 	data_set = get_data_set();
-	
-	ret = mgmt_msg_append(ret, data_set->transition_idle_timeout);
-	ret = mgmt_msg_append(ret, data_set->symmetric_cluster?"True":"False");
-	ret = mgmt_msg_append(ret, data_set->stonith_enabled?"True":"False");
-	
-	switch (data_set->no_quorum_policy) {
-		case no_quorum_freeze:
-			ret = mgmt_msg_append(ret, "freeze");
-			break;
-		case no_quorum_stop:
-			ret = mgmt_msg_append(ret, "stop");
-			break;
-		case no_quorum_ignore:
-			ret = mgmt_msg_append(ret, "ignore");
-			break;
+
+	ARGC_CHECK(2);
+
+	if (data_set == NULL){
+		return cl_strdup(MSG_FAIL);
+	}
+
+	if (STRNCMP_CONST(argv[1], "have_quorum") == 0){
+		ret = mgmt_msg_append(ret, data_set->have_quorum?"true":"false");
+		free_data_set(data_set);
+		return ret;
 	}
-	snprintf(buf, 255, "%d", data_set->default_resource_stickiness);
-	ret = mgmt_msg_append(ret, buf);
-	ret = mgmt_msg_append(ret, data_set->have_quorum?"True":"False");
-	snprintf(buf, 255, "%d", data_set->default_resource_fail_stickiness);
 	
-	ret = mgmt_msg_append(ret, buf);
+ 	if ( data_set->config_hash == NULL){
+		free_data_set(data_set);
+		return cl_strdup(MSG_FAIL);
+	}
+	value = g_hash_table_lookup(data_set->config_hash, argv[1]);
+
+	if (value == NULL){
+		ret = mgmt_msg_append(ret, "");
+	}
+	else{
+		ret = mgmt_msg_append(ret, value);
+	}	
+
 	free_data_set(data_set);
 	return ret;
 }
+
 char*
 on_update_crm_config(char* argv[], int argc)
 {
@@ -611,7 +698,7 @@ on_update_crm_config(char* argv[], int argc)
 			cur = g_list_next(cur);
 		}
 		rc = update_attr(cib_conn, cib_sync_call, XML_CIB_TAG_CRMCONFIG, NULL
-				, 		CIB_OPTIONS_FIRST, id, argv[1], argv[2]);
+				 , 		CIB_OPTIONS_FIRST, id, argv[1], argv[2], FALSE);
 	}
 	else {
 		crm_data_t* fragment = NULL;
@@ -621,7 +708,7 @@ on_update_crm_config(char* argv[], int argc)
 		
 		snprintf(xml, MAX_STRLEN, 
 			"<cluster_property_set id=\"cib-bootstrap-options\">"
-			"<attributes> <nvpair id=\"id-%s\"name=\"%s\" value=\"%s\"/>"
+			"<attributes> <nvpair id=\"cib-bootstrap-options-%s\"name=\"%s\" value=\"%s\"/>"
 			"</attributes> </cluster_property_set>", 
 			argv[1], argv[1], argv[2]);
 
@@ -936,7 +1023,7 @@ on_cleanup_rsc(char* argv[], int argc)
 	sleep(5); /* wait for the refresh */
 	now_s = crm_itoa(now);
 	update_attr(cib_conn, cib_sync_call,
-		    XML_CIB_TAG_CRMCONFIG, NULL, NULL, NULL, "last-lrm-refresh", now_s);
+		    XML_CIB_TAG_CRMCONFIG, NULL, NULL, NULL, "last-lrm-refresh", now_s, FALSE);
 	crm_free(now_s);
 
 	
@@ -956,81 +1043,119 @@ on_cleanup_rsc(char* argv[], int argc)
 	9	cmd += "\n"+rsc["clone_node_max"]
 	10	cmd += "\n"+rsc["master_max"]
 	11	cmd += "\n"+rsc["master_node_max"]
+	12	cmd += "\n"+rsc["new_group"]
 		for param in rsc["params"] :
-	12,15,18...	cmd += "\n"+param["id"]
-	13,16,19...	cmd += "\n"+param["name"]
-	14,17,20...	cmd += "\n"+param["value"]
+	13,16,19...	cmd += "\n"+param["id"]
+	14,17,20...	cmd += "\n"+param["name"]
+	15,18,21...	cmd += "\n"+param["value"]
 */
 char*
 on_add_rsc(char* argv[], int argc)
 {
-	int rc, i, in_group;
+	int rc, i, in_group, new_group;
 	crm_data_t* fragment = NULL;
 	crm_data_t* cib_object = NULL;
 	crm_data_t* output = NULL;
 	char xml[MAX_STRLEN];
 	char buf[MAX_STRLEN];
-	char inst_attrs_id[MAX_STRLEN];
+	char meta_attrs_id[MAX_STRLEN];
 	int clone, master, has_param;
 		
-	if (argc < 11) {
+	if (argc < 13) {
 		return cl_strdup(MSG_FAIL);
 	}
 	xml[0]=0;
 	in_group = (strlen(argv[5]) != 0);
+	new_group = (STRNCMP_CONST(argv[12], "True") == 0);
 	clone = (STRNCMP_CONST(argv[6], "clone") == 0);
 	master = (STRNCMP_CONST(argv[6], "master") == 0);
-	has_param = (argc > 11);
+	has_param = (argc > 13);
 	if (in_group) {
 		snprintf(buf, MAX_STRLEN, "<group id=\"%s\">", argv[5]);
 		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
+
+		if (new_group){
+			snprintf(buf, MAX_STRLEN,
+				"<meta_attributes id=\"%s_meta_attrs\"><attributes>" \
+				"<nvpair id=\"%s_metaattr_target_role\" name=\"target_role\" value=\"stopped\"/>" \
+				"</attributes> </meta_attributes>",
+				argv[5], argv[5]);
+			strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
+		}
+
+		
 	}
 	if (clone) {
-		get_instance_attributes_id(argv[7], inst_attrs_id);
+		get_meta_attributes_id(argv[7], meta_attrs_id);
 		snprintf(buf, MAX_STRLEN,
-			 "<clone id=\"%s\"><instance_attributes id=\"%s\"><attributes>" \
-			 "<nvpair id=\"%s_clone_max\" name=\"clone_max\" value=\"%s\"/>" \
-			 "<nvpair id=\"%s_clone_node_max\" name=\"clone_node_max\" value=\"%s\"/>" \
-			 "</attributes>	</instance_attributes> ",
-			 argv[7], inst_attrs_id, argv[7], argv[8],argv[7], argv[9]);
+			 "<clone id=\"%s\"><meta_attributes id=\"%s\"><attributes>" \
+			 "<nvpair id=\"%s_metaattr_target_role\" name=\"target_role\" value=\"stopped\"/>" \
+			 "<nvpair id=\"%s_metaattr_clone_max\" name=\"clone_max\" value=\"%s\"/>" \
+			 "<nvpair id=\"%s_metaattr_clone_node_max\" name=\"clone_node_max\" value=\"%s\"/>" \
+			 "</attributes>	</meta_attributes> ",
+			 argv[7], meta_attrs_id, argv[7], argv[7], argv[8],argv[7], argv[9]);
 		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
 	}
 	if (master) {
-		get_instance_attributes_id(argv[7], inst_attrs_id);
+		get_meta_attributes_id(argv[7], meta_attrs_id);
 		snprintf(buf, MAX_STRLEN,
-			 "<master_slave id=\"%s\"><instance_attributes id=\"%s\"><attributes>" \
-			 "<nvpair id=\"%s_clone_max\" name=\"clone_max\" value=\"%s\"/>" \
-			 "<nvpair id=\"%s_clone_node_max\" name=\"clone_node_max\" value=\"%s\"/>" \
-			 "<nvpair id=\"%s_master_max\" name=\"master_max\" value=\"%s\"/>" \
-			 "<nvpair id=\"%s_master_node_max\" name=\"master_node_max\" value=\"%s\"/>" \
-			 "</attributes>	</instance_attributes>",
-			 argv[7], inst_attrs_id, argv[7], argv[8], argv[7], argv[9],
-			 argv[7], argv[10], argv[7], argv[11]);
+			 "<master_slave id=\"%s\">" \
+			 "<meta_attributes id=\"%s\"><attributes>" \
+			 "<nvpair id=\"%s_metaattr_target_role\" name=\"target_role\" value=\"stopped\"/>" \
+			 "<nvpair id=\"%s_metaattr_clone_max\" name=\"clone_max\" value=\"%s\"/>" \
+			 "<nvpair id=\"%s_metaattr_clone_node_max\" name=\"clone_node_max\" value=\"%s\"/>" \
+			 "<nvpair id=\"%s_metaattr_master_max\" name=\"master_max\" value=\"%s\"/>" \
+			 "<nvpair id=\"%s_metaattr_master_node_max\" name=\"master_node_max\" value=\"%s\"/>" \
+			 "<nvpair id=\"%s_metaattr_notify\" name=\"notify\" value=\"true\"/>" \
+			 "<nvpair id=\"%s_metaattr_globally_unique\" name=\"globally_unique\" value=\"false\"/>" \
+			 "</attributes>	</meta_attributes>",
+			 argv[7], meta_attrs_id, argv[7], argv[7], argv[8], argv[7], argv[9],
+			 argv[7], argv[10], argv[7], argv[11], argv[7], argv[7]);
 		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
 	}
 	
-	if (!has_param) {
-		snprintf(buf, MAX_STRLEN,
-			 "<primitive id=\"%s\" class=\"%s\" type=\"%s\" provider=\"%s\"/>"
-					 , argv[1],argv[2], argv[3],argv[4]);
-		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
+	if ((in_group || clone || master) && !has_param){
+			snprintf(buf, MAX_STRLEN,
+				 "<primitive id=\"%s\" class=\"%s\" type=\"%s\" provider=\"%s\"/>"
+						 , argv[1],argv[2], argv[3],argv[4]);
+			strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
 	}
 	else {
 		snprintf(buf, MAX_STRLEN,
-			 "<primitive id=\"%s\" class=\"%s\" type=\"%s\" provider=\"%s\">" \
+			 "<primitive id=\"%s\" class=\"%s\" type=\"%s\" provider=\"%s\">",
+			 argv[1],argv[2], argv[3],argv[4]);
+		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
+	}
+
+	if (!in_group && !clone && !master){
+		snprintf(buf, MAX_STRLEN,
+			"<meta_attributes id=\"%s_meta_attrs\"> <attributes>" \
+			"<nvpair id=\"%s_metaattr_target_role\" name=\"target_role\" value=\"stopped\"/>" \
+			"</attributes> </meta_attributes>",
+			argv[1], argv[1]);
+		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
+	}
+
+	if (has_param) {
+		snprintf(buf, MAX_STRLEN,
 			 "<instance_attributes id=\"%s_instance_attrs\"> <attributes>"
-			 , argv[1],argv[2], argv[3],argv[4], argv[1]);
+			 , argv[1]);
 		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
 	
-		for (i = 12; i < argc; i += 3) {
+		for (i = 13; i < argc; i += 3) {
 			snprintf(buf, MAX_STRLEN,
 				 "<nvpair id=\"%s\" name=\"%s\" value=\"%s\"/>",
 				 argv[i], argv[i+1],argv[i+2]);
 			strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
 		}
-		strncat(xml, "</attributes></instance_attributes></primitive>",
+		strncat(xml, "</attributes></instance_attributes>",
 				sizeof(xml)-strlen(xml)-1);
 	}
+
+	if (has_param || (!in_group && !clone && !master)){
+		strncat(xml, "</primitive>", sizeof(xml)-strlen(xml)-1);
+	}
+
 	if (master) {
 		strncat(xml, "</master_slave>", sizeof(xml)-strlen(xml)-1);
 	}
@@ -1217,7 +1342,7 @@ on_get_all_rsc(char* argv[], int argc)
 	cur = data_set->resources;
 	while (cur != NULL) {
 		resource_t* rsc = (resource_t*)cur->data;
-		if(rsc->orphan == FALSE || rsc->role != RSC_ROLE_STOPPED) {
+		if(is_not_set(rsc->flags, pe_rsc_orphan) || rsc->role != RSC_ROLE_STOPPED) {
 			ret = mgmt_msg_append(ret, rsc->id);
 		}
 		cur = g_list_next(cur);
@@ -1318,11 +1443,11 @@ on_get_rsc_status(char* argv[], int argc)
 			ret = mgmt_msg_append(ret, "unknown");
 			break;
 		case pe_native:
-			if(rsc->is_managed == FALSE) {
+			if(is_not_set(rsc->flags, pe_rsc_managed)) {
 				ret = mgmt_msg_append(ret, "unmanaged");
 				break;
 			}
-			if( rsc->failed ) {
+			if(is_set(rsc->flags, pe_rsc_failed)) {
 				ret = mgmt_msg_append(ret, "failed");
 				break;
 			}
@@ -1414,6 +1539,43 @@ on_get_sub_rsc(char* argv[], int argc)
 	return ret;
 }
 
+
+char*
+on_get_rsc_metaattrs(char* argv[], int argc)
+{
+	int i;
+	resource_t* rsc;
+	char* ret;
+	struct ha_msg* attrs;
+	struct ha_msg* nvpair;
+	pe_working_set_t* data_set;
+	
+	data_set = get_data_set();
+	GET_RESOURCE()
+
+	ret = cl_strdup(MSG_OK);
+	attrs = cl_get_struct((struct ha_msg*)rsc->xml, "meta_attributes");
+	if(attrs == NULL) {
+		free_data_set(data_set);
+		return ret;
+	}
+	attrs = cl_get_struct(attrs, "attributes");
+	if(attrs == NULL) {
+		free_data_set(data_set);
+		return ret;
+	}
+	for (i = 0; i < attrs->nfields; i++) {
+		if (STRNCMP_CONST(attrs->names[i], "nvpair") == 0) {
+			nvpair = (struct ha_msg*)attrs->values[i];
+			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "id"));
+			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "name"));
+			ret = mgmt_msg_append(ret, ha_msg_value(nvpair, "value"));
+		}
+	}
+	free_data_set(data_set);
+	return ret;
+}
+
 /* resource params */
 char*
 on_get_rsc_params(char* argv[], int argc)
@@ -1513,6 +1675,56 @@ on_update_rsc_attr(char* argv[], int argc)
 	return cl_strdup(MSG_OK);
 }
 
+char*
+on_update_rsc_metaattrs(char* argv[], int argc)
+{
+	int rc, i;
+	crm_data_t* fragment = NULL;
+	crm_data_t* cib_object = NULL;
+	crm_data_t* output;
+	char xml[MAX_STRLEN];
+	char buf[MAX_STRLEN];
+	char prefix[MAX_STRLEN];
+	char suffix[MAX_STRLEN];
+	char real_id[MAX_STRLEN];
+	char meta_attrs_id[MAX_STRLEN];	
+	
+	if(get_fix(argv[1], prefix, suffix, real_id) == -1) {
+		return cl_strdup(MSG_FAIL);
+	}
+	get_meta_attributes_id(argv[1],meta_attrs_id);
+	snprintf(xml, MAX_STRLEN,
+    		 "%s<meta_attributes id=\"%s\"><attributes>",
+    		 prefix , meta_attrs_id);
+	for (i = 2; i < argc; i += 3) {
+		snprintf(buf, MAX_STRLEN,
+			"<nvpair id=\"%s\" name=\"%s\" value=\"%s\"/>",
+			argv[i], argv[i+1], argv[i+2]);
+		strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
+	}
+	strncat(xml, "</attributes></meta_attributes>",
+			sizeof(xml)-strlen(xml)-1);
+	strncat(xml, suffix, sizeof(xml)-strlen(xml)-1);
+
+	cib_object = string2xml(xml);
+	if(cib_object == NULL) {
+		return cl_strdup(MSG_FAIL);
+	}
+	mgmt_log(LOG_INFO, "on_update_rsc_metaattrs:%s",xml);
+	fragment = create_cib_fragment(cib_object, "resources");
+
+	rc = cib_conn->cmds->update(
+			cib_conn, "resources", fragment, &output, cib_sync_call);
+
+	free_xml(fragment);
+	free_xml(cib_object);
+	if (rc < 0) {
+		return crm_failed_msg(output, rc);
+	}
+	free_xml(output);
+	return cl_strdup(MSG_OK);
+}
+
 char*
 on_update_rsc_params(char* argv[], int argc)
 {
@@ -1562,6 +1774,19 @@ on_update_rsc_params(char* argv[], int argc)
 	free_xml(output);
 	return cl_strdup(MSG_OK);
 }
+
+char*
+on_delete_rsc_metaattr(char* argv[], int argc)
+{
+	crm_data_t * output;
+	int rc;
+	ARGC_CHECK(2)
+
+	if ((rc=delete_object("resources", "nvpair", argv[1], &output)) < 0) {
+		return crm_failed_msg(output, rc);
+	}
+	return cl_strdup(MSG_OK);
+}
 char*
 on_delete_rsc_param(char* argv[], int argc)
 {
@@ -1587,14 +1812,22 @@ on_set_target_role(char* argv[], int argc)
 	char prefix[MAX_STRLEN];
 	char suffix[MAX_STRLEN];
 	char real_id[MAX_STRLEN];
-	char inst_attrs_id[MAX_STRLEN];	
+	char meta_attrs_id[MAX_STRLEN];	
 	char target_role_id[MAX_STRLEN];	
 	
 	if(get_fix(argv[1], prefix, suffix, real_id) == -1) {
 		return cl_strdup(MSG_FAIL);
 	}
-	get_instance_attributes_id(argv[1], inst_attrs_id);
-	get_attr_id(argv[1], "target_role", target_role_id);
+
+	get_attr_id(argv[1], "instance_attributes", "target_role", target_role_id);
+	snprintf(buf, MAX_STRLEN, "%s", target_role_id);
+	rc = delete_object("resources", "nvpair", buf, &output);
+	if (rc < 0) {
+		return crm_failed_msg(output, rc);
+	}
+	
+	get_meta_attributes_id(argv[1], meta_attrs_id);
+	get_attr_id(argv[1], "meta_attributes", "target_role", target_role_id);
 	
 	if (STRNCMP_CONST(argv[2],"#default") == 0) {
 		snprintf(buf, MAX_STRLEN, "%s", target_role_id);
@@ -1607,15 +1840,15 @@ on_set_target_role(char* argv[], int argc)
 		
 
 	snprintf(xml, MAX_STRLEN,
-    		 "%s<instance_attributes id=\"%s\"><attributes>",
-    		 prefix,inst_attrs_id);
+    		 "%s<meta_attributes id=\"%s\"><attributes>",
+    		 prefix,meta_attrs_id);
 	snprintf(buf, MAX_STRLEN,
 		"<nvpair id=\"%s\" " \
 		"name=\"target_role\" value=\"%s\"/>",
 		target_role_id, argv[2]);
 	strncat(xml, buf, sizeof(xml)-strlen(xml)-1);
 	
-	strncat(xml, "</attributes></instance_attributes>",
+	strncat(xml, "</attributes></meta_attributes>",
 			sizeof(xml)-strlen(xml)-1);
 	strncat(xml, suffix, sizeof(xml)-strlen(xml)-1);
 
@@ -1889,17 +2122,17 @@ on_update_clone(char* argv[], int argc)
 	crm_data_t* cib_object = NULL;
 	crm_data_t* output;
 	char xml[MAX_STRLEN];
-	char inst_attrs_id[MAX_STRLEN];	
+	char meta_attrs_id[MAX_STRLEN];	
 
 	ARGC_CHECK(4);
 	
-	get_instance_attributes_id(argv[1], inst_attrs_id);
+	get_meta_attributes_id(argv[1], meta_attrs_id);
 	snprintf(xml,MAX_STRLEN,
-		 "<clone id=\"%s\"><instance_attributes id=\"%s\"><attributes>" \
+		 "<clone id=\"%s\"><meta_attributes id=\"%s\"><attributes>" \
 		 "<nvpair id=\"%s_clone_max\" name=\"clone_max\" value=\"%s\"/>" \
 		 "<nvpair id=\"%s_clone_node_max\" name=\"clone_node_max\" value=\"%s\"/>" \
-		 "</attributes></instance_attributes></clone>",
-		 argv[1],inst_attrs_id,argv[1],argv[2],argv[1],argv[3]);
+		 "</attributes></meta_attributes></clone>",
+		 argv[1],meta_attrs_id,argv[1],argv[2],argv[1],argv[3]);
 
 	cib_object = string2xml(xml);
 	if(cib_object == NULL) {
@@ -1970,18 +2203,18 @@ on_update_master(char* argv[], int argc)
 	crm_data_t* cib_object = NULL;
 	crm_data_t* output;
 	char xml[MAX_STRLEN];
-	char inst_attrs_id[MAX_STRLEN];	
+	char meta_attrs_id[MAX_STRLEN];	
 
 	ARGC_CHECK(6);
-	get_instance_attributes_id(argv[1], inst_attrs_id);
+	get_meta_attributes_id(argv[1], meta_attrs_id);
 	snprintf(xml,MAX_STRLEN,
-		 "<master_slave id=\"%s\"><instance_attributes id=\"%s\"><attributes>" \
+		 "<master_slave id=\"%s\"><meta_attributes id=\"%s\"><attributes>" \
 		 "<nvpair id=\"%s_clone_max\" name=\"clone_max\" value=\"%s\"/>" \
 		 "<nvpair id=\"%s_clone_node_max\" name=\"clone_node_max\" value=\"%s\"/>" \
 		 "<nvpair id=\"%s_master_max\" name=\"master_max\" value=\"%s\"/>" \
 		 "<nvpair id=\"%s_master_node_max\" name=\"master_node_max\" value=\"%s\"/>" \
-		 "</attributes></instance_attributes></master_slave>",
-		 argv[1],inst_attrs_id,argv[1],argv[2],argv[1],
+		 "</attributes></meta_attributes></master_slave>",
+		 argv[1],meta_attrs_id,argv[1],argv[2],argv[1],
 		 argv[3],argv[1],argv[4],argv[1],argv[5]);
 
 	cib_object = string2xml(xml);
@@ -2065,6 +2298,7 @@ on_get_constraint(char* argv[], int argc)
 				ret = mgmt_msg_append(ret, ha_msg_value(constraint, "rsc"));
 				rule = find_xml_node(constraint,"rule",TRUE);
 				ret = mgmt_msg_append(ret, ha_msg_value(rule, "score"));
+				ret = mgmt_msg_append(ret, ha_msg_value(rule, "boolean_op"));
 				expr_list = find_xml_node_list(rule, "expression");
 				expr_cur = expr_list;
 				while(expr_cur) {
@@ -2123,13 +2357,13 @@ on_update_constraint(char* argv[], int argc)
 	if (STRNCMP_CONST(argv[1],"rsc_location")==0) {
 		snprintf(xml, MAX_STRLEN,
 			 "<rsc_location id=\"%s\" rsc=\"%s\">" \
-				"<rule id=\"prefered_%s\" score=\"%s\">",
-		 	 argv[2], argv[3], argv[2], argv[4]);
-		for (i = 0; i < (argc-5)/4; i++) {
+				"<rule id=\"prefered_%s\" score=\"%s\" boolean_op=\"%s\">",
+		 	 argv[2], argv[3], argv[2], argv[4], argv[5]);
+		for (i = 0; i < (argc-6)/4; i++) {
 			char expr[MAX_STRLEN];
 			snprintf(expr, MAX_STRLEN,
 				 "<expression attribute=\"%s\" id=\"%s\" operation=\"%s\" value=\"%s\"/>",
-			 	 argv[5+i*4+1],argv[5+i*4],argv[5+i*4+2],argv[5+i*4+3]);
+			 	 argv[6+i*4+1],argv[6+i*4],argv[6+i*4+2],argv[6+i*4+3]);
 			strncat(xml, expr, sizeof(xml)-strlen(xml)-1);
 		}
 		strncat(xml, "</rule></rsc_location>",
