@@ -24,6 +24,7 @@
 #include <lha_internal.h>
 
 #include "hbagent.h"
+#include "hbagentv2.h"
 
 #include "hb_api.h"
 #include "heartbeat.h"
@@ -66,13 +67,12 @@
 #include <errno.h>
 
 #include "saf/ais.h"
-
-#define DEFAULT_TIME_OUT 5 /* default timeout value for snmp in sec. */
-#define LHAAGENTID "lha-snmpagent"
+#include "clplumbing/cl_uuid.h" /* UU_UNPARSE_SIZEOF */
 
 static unsigned long hbInitialized = 0;
 static ll_cluster_t * hb = NULL; /* heartbeat handle */
-static char * myid = NULL; /* my node id */
+char * myid = NULL; /* my node id */
+char * myuuid = NULL; /* my node uuid */
 static SaClmHandleT clm = 0;
 static unsigned long clmInitialized = 0;
 
@@ -82,6 +82,7 @@ static GPtrArray * gMembershipTable = NULL;
 static GPtrArray * gResourceTable = NULL;
 
 static int keep_running;
+int snmp_cache_time_out = CACHE_TIME_OUT;
 
 int init_heartbeat(void);
 int get_heartbeat_fd(void);
@@ -107,6 +108,35 @@ int hbagent_trap(int online, const char * node);
 
 int ping_membership(int * mem_fd);
 
+/* LHAHeartbeatConfigInfo partial-mode */
+#define DEFAULT_REFRESH_TIMING  (0)
+
+int  init_hbconfig(void);
+void free_hbconfig(void);
+int  hbconfig_get_str_value(const char * attr, char * * value);
+
+static int    hbconfig_partial = 0;
+static char * hbconfig[LHA_CONF_END];
+static struct hbconfig_map {
+	lha_hbconfig_t  attr_no;
+	const char *    attr;
+} hbconfig_map[] =
+{
+	{LHA_CONF_HBVERSION,   KEY_HBVERSION},
+	{LHA_CONF_KEEPALIVE,   KEY_KEEPALIVE},
+	{LHA_CONF_DEADTIME,    KEY_DEADTIME},
+	{LHA_CONF_DEADPING,    KEY_DEADPING},
+	{LHA_CONF_WARNTIME,    KEY_WARNTIME},
+	{LHA_CONF_INITDEAD,    KEY_INITDEAD},
+	{LHA_CONF_BAUDRATE,    KEY_BAUDRATE},
+	{LHA_CONF_AUTOFAIL,    KEY_AUTOFAIL},
+	{LHA_CONF_STONITH,     KEY_STONITH},
+	{LHA_CONF_STONITHHOST, KEY_STONITHHOST},
+	{LHA_CONF_RESPAWN,     KEY_CLIENT_CHILD},
+	{LHA_CONF_END,         NULL}
+};
+
+
 uint32_t get_status_value(const char * status, const char * * status_array, uint32_t * value_array);
 
 const char * NODE_STATUS [] = 
@@ -280,6 +310,73 @@ clusterinfo_get_int_value(lha_attribute_t attr, size_t index, uint32_t * value)
     return HA_OK;
 }
 
+/* LHAHeartbeatConfigInfo partial-mode */
+int
+init_hbconfig(void)
+{
+    lha_hbconfig_t      attr_no;
+    char *              value;
+
+    for (attr_no = LHA_CONF_HBVERSION; attr_no < LHA_CONF_END; ++attr_no) {
+
+        hbconfig[attr_no] = NULL;
+
+        if(!hbconfig_partial)
+            continue;
+
+        if (hbconfig_get_str_value(hbconfig_map[attr_no].attr, &value)
+                                                                != HA_OK) {
+            cl_log(LOG_INFO, "LHAHeartbeatConfigInfo initialization error.  "
+                             "maybe get_parameter returned NULL.");
+            continue;
+        }
+
+        hbconfig[attr_no] = value;
+    }
+
+    return HA_OK;
+}
+
+void
+free_hbconfig(void)
+{
+    lha_hbconfig_t      attr_no;
+
+    if (!hbconfig_partial)
+        return;
+
+    for (attr_no = LHA_CONF_HBVERSION; attr_no < LHA_CONF_END; ++attr_no) {
+
+        if (hbconfig[attr_no] != NULL)
+            cl_free(hbconfig[attr_no]);
+
+    }
+}
+
+int
+hbconfig_get_str(lha_hbconfig_t attr_no, char * * value)
+{
+    static char err[] = "N/A";
+
+    if (hbconfig[attr_no] != NULL) {
+
+        *value = cl_strdup(hbconfig[attr_no]);
+
+    } else {
+
+        if (hbconfig_get_str_value(hbconfig_map[attr_no].attr, value)
+                                                                != HA_OK) {
+            cl_log(LOG_INFO, "LHAHeartbeatConfigInfo getting parameter error.  "
+                             "maybe get_parameter returned NULL.");
+            *value  = cl_strdup(err);
+        }
+
+    }
+
+    return HA_OK;
+}
+
+
 int
 hbconfig_get_str_value(const char * attr, char * * value)
 {
@@ -298,7 +395,7 @@ hbconfig_get_str_value(const char * attr, char * * value)
 	return HA_OK;
     };
 
-    *value = cl_strdup(ret);
+    *value = ret;
 
     return HA_OK;
 }
@@ -442,31 +539,35 @@ void
 free_storage(void)
 {
     	free_nodetable();
-	g_ptr_array_free(gNodeTable, 0);
+	g_ptr_array_free(gNodeTable, 1);
 	gNodeTable = NULL;
 
 	free_iftable();
-	g_ptr_array_free(gIFTable, 0);
+	g_ptr_array_free(gIFTable, 1);
 	gIFTable = NULL;
 
 	free_resourcetable();
-	g_ptr_array_free(gResourceTable, 0);
+	g_ptr_array_free(gResourceTable, 1);
 	gResourceTable = NULL;
 
 	free_membershiptable();
-	g_ptr_array_free(gMembershipTable, 0);
-	gResourceTable = NULL;
+	g_ptr_array_free(gMembershipTable, 1);
+	gMembershipTable = NULL;
 
 }
 
 int
 init_heartbeat(void)
 {
-	const char * parameter;
+	char * parameter;
+	cl_uuid_t uuid;
 	hb = NULL;
 
 	cl_log_set_entity("lha-snmpagent");
-	cl_log_set_facility(LOG_USER);
+	cl_log_set_facility(HA_LOG_FACILITY);
+
+	/* Use logd if it's enabled by heartbeat */
+	cl_inherit_logging_environment(0);
 
 	hb = ll_cluster_new("heartbeat");
 
@@ -483,6 +584,7 @@ init_heartbeat(void)
 	parameter = hb->llc_ops->get_parameter(hb, KEY_COREROOTDIR);
 	if (parameter) {
 		cl_set_corerootdir(parameter);
+		cl_free(parameter);
 	}
 	cl_cdtocoredir();
 
@@ -492,6 +594,18 @@ init_heartbeat(void)
 		return HA_FAIL;
 	}
 
+	/*
+	 * get uuid for trap message.
+	 *   see: hbagentv2_update_diff() in hbagentv2.c
+	 */
+	if (hb->llc_ops->get_uuid_by_name(hb, myid, &uuid) == HA_FAIL) {
+		cl_log(LOG_ERR, "Cannot get mynodeid");
+		cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
+		return HA_FAIL;
+	}
+	myuuid = cl_malloc(UU_UNPARSE_SIZEOF);
+	cl_uuid_unparse(&uuid, myuuid);
+
 	if (hb->llc_ops->set_nstatus_callback(hb, NodeStatus, NULL) !=HA_OK){
 	        cl_log(LOG_ERR, "Cannot set node status callback");
 	        cl_log(LOG_ERR, "REASON: %s", hb->llc_ops->errmsg(hb));
@@ -1190,7 +1304,15 @@ membership_trap(const char * node, SaClmClusterChangesT status)
 static void
 usage(void)
 {
-    	fprintf(stderr, "Usage: hbagent -d\n");
+    	fprintf(stderr, "Usage: hbagent [Options]\n"
+    			"Options:\n"
+    			" -d         enable logging to STDERR\n"
+    			" -r SECOND  set LHAHeartbeatConfigInfo to partial-mode\n"
+    			"            SECOND -- time interval between update "
+    			"LHAHeartbeatConfigInfo\n"
+    			"                      o multiple of 5\n"
+    			"                      o '0' keeps initial value (not update)\n"
+    	);
 }
 
 int
@@ -1202,6 +1324,7 @@ main(int argc, char ** argv)
 	struct timeval tv, *tvp;
 	int flag, block = 0, numfds, hb_fd = 0, mem_fd = 0, debug = 0;
 	int hb_already_dead = 0;
+	int cib_fd = 0;
 
 	/* change this if you want to be a SNMP master agent */
 	int agentx_subagent=1; 
@@ -1209,30 +1332,42 @@ main(int argc, char ** argv)
 	/* change this if you want to run in the background */
 	int background = 0; 
 
-	/* change this if you want to use syslog */
-	int syslog = 1; 
+	/* LHAHeartbeatConfigInfo partial-mode */
+	int hbconfig_refresh_timing = 0;
+	int hbconfig_refresh_cnt;
+
+	while ((flag = getopt(argc, argv, "dr:h")) != EOF) {
+	    	int i;
 
-	while ((flag = getopt(argc, argv, "d")) != EOF) {
 	    	switch (flag) {
 		    case 'd':
 			debug++ ;
 			break;
+		    case 'r': /* LHAHeartbeatConfigInfo partial-mode */
+			hbconfig_partial = 1;
+			i = atoi(optarg);
+			if (i > 0)
+				hbconfig_refresh_timing = i / DEFAULT_TIME_OUT;
+			else
+				hbconfig_refresh_timing = DEFAULT_REFRESH_TIMING;
+			snmp_cache_time_out = i;
+			break;
+		    case 'h':
+			usage();
+			exit(0);
 		    default: 
 			usage();
 			exit(1);
 		}
 	}
 
-	if (debug) 
+	if (debug) {
 		cl_log_enable_stderr(TRUE);
-	else 
-		cl_log_enable_stderr(FALSE);
-
-	/* print log errors to syslog or stderr */
-	if (syslog)
-		snmp_enable_calllog();
-	else
 		snmp_enable_stderrlog();
+	} else {
+		cl_log_enable_stderr(FALSE);
+		snmp_enable_calllog();
+	}
 
 	/* we're an agentx subagent? */
 	if (agentx_subagent) {
@@ -1242,7 +1377,7 @@ main(int argc, char ** argv)
 	}
 
 	/* run in background, if requested */
-	if (background && netsnmp_daemonize(1, !syslog))
+	if (background && netsnmp_daemonize(1, debug))
 		exit(1);
 
 	/* initialize the agent library */
@@ -1281,6 +1416,11 @@ main(int argc, char ** argv)
 	} 
 	*/
 
+	/* LHAHeartbeatConfigInfo partial-mode */
+	if ((ret = init_hbconfig()) != HA_OK) {
+	    	return -3;
+	}
+
 	init_LHAClusterInfo();
 	init_LHANodeTable();
 	init_LHAIFStatusTable();
@@ -1288,6 +1428,13 @@ main(int argc, char ** argv)
 	init_LHAMembershipTable();
 	init_LHAHeartbeatConfigInfo();
 
+
+	/* now implementing: hbagentv2 */
+	if ((ret = init_hbagentv2()) != HA_OK ||
+	    (cib_fd = get_cib_fd()) <= 0) {
+		return -4;
+	}
+
 	/* LHA-agent will be used to read LHA-agent.conf files. */
 	init_snmp("LHA-agent");
 
@@ -1304,6 +1451,8 @@ main(int argc, char ** argv)
 
 	hbagent_trap(1, myid);
 
+	hbconfig_refresh_cnt = 0;
+
 	/* you're main loop here... */
 	while(keep_running) {
 	/* if you use select(), see snmp_select_info() in snmp_api(3) */
@@ -1321,6 +1470,10 @@ main(int argc, char ** argv)
 			if (mem_fd > hb_fd)
 				numfds = mem_fd + 1;
 		}
+		FD_SET(cib_fd, &fdset);
+		if (numfds < (cib_fd + 1)) {
+			numfds = cib_fd + 1;
+		}
 
 		tv.tv_sec = DEFAULT_TIME_OUT;
 		tv.tv_usec = 0;
@@ -1358,6 +1511,14 @@ main(int argc, char ** argv)
 		} else if (ret == 0) {
 			/* timeout */
 			ping_membership(&mem_fd);
+
+			/* LHAHeartbeatConfigInfo partial-mode */
+			if (hbconfig_refresh_timing
+			&& ++hbconfig_refresh_cnt >= hbconfig_refresh_timing) {
+				free_hbconfig();
+				init_hbconfig();
+				hbconfig_refresh_cnt = 0;
+			}
 			snmp_timeout();
 			goto process_pending;
 		} 
@@ -1377,6 +1538,12 @@ main(int argc, char ** argv)
 			    	cl_log(LOG_DEBUG, "unrecoverable membership error. quit now.");
 				break;
 			}
+		} else  if (FD_ISSET(cib_fd, &fdset)) {
+			/* change cib info events */
+			if ((ret = handle_cib_msg()) == HA_FAIL) {
+		  		cl_log(LOG_DEBUG, "unrecoverable CIB error. quit now.");
+				break;
+			}
 		} else {
 
 			/* snmp request */
@@ -1394,7 +1561,10 @@ process_pending:
 	hbagent_trap(0, myid);
 	snmp_shutdown("LHA-agent");
 
+	free_hbagentv2();
+ 	free_hbconfig();
 	cl_free(myid);
+	cl_free(myuuid);
 	free_storage();
 
         if (!hb_already_dead && hb->llc_ops->signoff(hb, TRUE) != HA_OK) {
@@ -1411,4 +1581,3 @@ process_pending:
 	return 0;
 }
 
-
