@@ -1,22 +1,21 @@
-/* $Id: recoverymgrd.c,v 1.8.2.2 2004/10/16 05:22:46 alan Exp $ */
 /*
  * Generic Recovery manager implementation
  * 
  * Copyright (c) 2002 Intel Corporation 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- *  version 2.1 of the License, or (at your option) any later version.
  *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
+ * General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
- USA 
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * This is a generic implementation of a recovery manager.
  * For the most basic case, the recovery manager will handle 
@@ -29,7 +28,7 @@
  * 
  */
 
-#include <portability.h>
+#include <lha_internal.h>
 #include <sys/time.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -54,6 +53,7 @@
 #include <clplumbing/coredumps.h>
 #include <apphb_notify.h>
 #include <recoverymgr.h>
+#include <clplumbing/cl_pidfile.h>
 
 #include "recoverymgrd.h"
 #include "configfile.h"
@@ -63,7 +63,7 @@
 #define HBINTERVAL_MSEC		2000
 
 #define CONFIG_FILE	"./recoverymgrd.conf"
-
+#define PIDFILE		HA_VARRUNDIR "/recoverymgrd.pid"
 #define DEBUG 
 #define         DBGMIN          1
 #define         DBGDETAIL       3
@@ -97,7 +97,7 @@ print_hash(gpointer key, gpointer value, gpointer userdata)
 {
 	char* key_str = key;
 	char* value_str = value;
-	cl_log(LOG_INFO, "key[%s], value[%s]\n", key_str, value_str);
+	cl_log(LOG_INFO, "key[%s], value[%s]", key_str, value_str);
 }
 
 gboolean
@@ -130,6 +130,7 @@ main(int argc, char *argv[])
    	int rc; 
    	int retval = 0;  
    	const char* conf_file = CONFIG_FILE;
+	int pid;
 	cl_cdtocoredir();
 	
 	if(argc == 2){
@@ -138,9 +139,17 @@ main(int argc, char *argv[])
 		printf("Usage: %s [config_file]\n", cmdname);
 		exit(LSB_EXIT_NOTCONFIGED);
 	}
+
+   	cl_log_enable_stderr(TRUE);
+
+	pid = cl_read_pidfile(PIDFILE);
+	if (pid > 0 && pid != getpid()){
+		cl_log(LOG_INFO, "recovermgrd is already running[%d]",
+		       pid);
+		return 0;
+	}
    	
 	cl_log_set_entity(argv[0]);
-   	cl_log_enable_stderr(TRUE);
    	cl_log_set_facility(LOG_USER);
    	cl_log(LOG_INFO, "Starting %s", argv[0]);
    	signal(SIGCHLD, sigchld_handler);
@@ -156,6 +165,13 @@ main(int argc, char *argv[])
    	printf("Debug mode -- non daemon\n");
 #endif
 
+	if (cl_lock_pidfile(PIDFILE) < 0){
+		
+		cl_log(LOG_INFO, "recoverymgrd is already running[%d]",
+		       cl_read_pidfile(PIDFILE));
+		return 0;
+	}
+
    	/* register with apphbd as a client and send pings */
    	retval = register_hb();
    	if (0 != retval)
@@ -169,11 +185,12 @@ main(int argc, char *argv[])
    	/* unregister and shutdown */
    	rc = apphb_unregister();
    	if (rc < 0) 
-   	{
+		{
       		cl_perror("apphb_unregister failure");
       		exit(3);
    	}
-
+	
+	cl_unlock_pidfile(PIDFILE);
    	return 0; 
 }
 
@@ -306,7 +323,7 @@ pending_conn_dispatch(IPC_Channel* src, gpointer user)
 
         if (debug >= DBGMIN) 
 	{
-		cl_log(LOG_DEBUG,"received connection request\n");
+		cl_log(LOG_DEBUG,"received connection request");
                 cl_log(LOG_DEBUG, "recoverymgr dispatch: IPC_channel: 0x%x"
                 " pid=%d"
                 ,       GPOINTER_TO_UINT(src)
@@ -340,6 +357,8 @@ recoverymgr_client_new(struct IPC_CHANNEL* ch)
 
         ret = g_new(recoverymgr_client_t, 1);
 
+	memset(ret, 0, sizeof(*ret));
+	
         ret->appname = NULL;
         ret->appinst = NULL;
         ret->ch = ch;
@@ -410,10 +429,10 @@ void
 recoverymgr_client_remove(gpointer Client)
 {
         recoverymgr_client_t* client = Client;
-        cl_log(LOG_INFO, "recoverymgr_client_remove: client: %ld\n"
+        cl_log(LOG_INFO, "recoverymgr_client_remove: client: %ld"
         ,       (long)client->pid);
         if (debug >= DBGMIN) {
-                cl_log(LOG_DEBUG, "recoverymgr_client_remove: client pid: %ld\n"
+                cl_log(LOG_DEBUG, "recoverymgr_client_remove: client pid: %ld"
                 ,       (long)client->pid);
         }
         G_main_del_IPC_Channel(client->source);
@@ -556,7 +575,7 @@ recoverymgr_client_connect(recoverymgr_client_t *client, void *Msg, size_t lengt
         IPC_Auth*               	clientauth;*/
 
         if (debug >= DBGDETAIL) {
-		cl_log(LOG_DEBUG, "recoverymgr_client_connect\n");
+		cl_log(LOG_DEBUG, "recoverymgr_client_connect");
 	}
 
         if (client->appname) {
@@ -614,7 +633,7 @@ static int
 recoverymgr_client_disconnect(recoverymgr_client_t* client , void * msg, size_t msgsize)
 {
         if (debug >= DBGDETAIL) {
-		cl_log(LOG_DEBUG, "recoverymgr_client_disconnect\n");
+		cl_log(LOG_DEBUG, "recoverymgr_client_disconnect");
 	}
         client->deleteme=TRUE;
         return 0;
@@ -646,7 +665,7 @@ recoverymgr_client_event(recoverymgr_client_t *client, void *Msg, size_t msgsize
 
         if (debug >= DBGDETAIL) 
 	{
-		cl_log(LOG_CRIT,"recoverymgr_client_event: event=%d\n", msg->event);
+		cl_log(LOG_CRIT,"recoverymgr_client_event: event=%d", msg->event);
 	}
 
 	if (msgsize < sizeof(*msg))
@@ -673,13 +692,13 @@ recoverymgr_client_event(recoverymgr_client_t *client, void *Msg, size_t msgsize
 	info = g_hash_table_lookup(scripts, msg->appname);
 	if (NULL == info)	
  	{
-		cl_log(LOG_INFO, "No script available to recover %s\n", msg->appname);
+		cl_log(LOG_INFO, "No script available to recover %s", msg->appname);
 		return 0;
 	}
 
 	if (info->event[msg->event].inuse == FALSE)
  	{
-		cl_log(LOG_INFO,"Script does not handle this event\n");
+		cl_log(LOG_INFO,"Script does not handle this event");
 
 		return 0;
 	}
@@ -702,7 +721,7 @@ recover_app(RecoveryInfo *info, int eventindex)
 	pid = fork();
 	if (pid < 0)
 	{
-		cl_log(LOG_CRIT,"Failed to fork recovery process\n");
+		cl_log(LOG_CRIT,"Failed to fork recovery process");
 		return -1;
 	}
 	else if (0 == pid)
@@ -712,16 +731,17 @@ recover_app(RecoveryInfo *info, int eventindex)
 		{
    			cl_log(LOG_INFO, "current euid[%ld]", (long)geteuid());
    			cl_log(LOG_INFO, "current egid[%ld]", (long)getegid());
-			cl_log(LOG_DEBUG,"script = %s\n", info->scriptname);
-			cl_log(LOG_DEBUG,"args = %s\n", info->event[eventindex].args);
+			cl_log(LOG_DEBUG,"script = %s", info->scriptname);
+			cl_log(LOG_DEBUG,"args = %s", info->event[eventindex].args);
 		}
 	
-		if (eventindex > MAXEVENTS) eventindex = 0;
-
+		if (eventindex > MAXEVENTS) {
+			eventindex = 0;
+		}
  	 	if (execl(info->scriptname, info->scriptname,
 			  info->event[eventindex].args, (const char *)NULL) < 0)
 		{
-			cl_perror("Failed to exec recovery script for %s\n", info->appname);
+			cl_perror("Failed to exec recovery script for %s", info->appname);
 			_exit(EXIT_FAILURE);
 		}
 	}
@@ -741,7 +761,7 @@ child_setup_function(RecoveryInfo *info)
    	{
         	if (debug >= DBGDETAIL) 
 		{
-      			cl_log(LOG_INFO,"Unable to get app data for recovery.\n");
+      			cl_log(LOG_INFO,"Unable to get app data for recovery.");
 		}
       		return;
    	}
@@ -751,7 +771,7 @@ child_setup_function(RecoveryInfo *info)
         	if (debug >= DBGDETAIL) 
 		{
 			cl_log(LOG_INFO, "error:[%s]", strerror(errno));
-      			cl_log(LOG_INFO, "Unable to setgid for recovery of %s\n", info->appname);
+      			cl_log(LOG_INFO, "Unable to setgid for recovery of %s", info->appname);
 		}
    	}
 	if ( 0 != setuid(info->uid))
@@ -759,7 +779,7 @@ child_setup_function(RecoveryInfo *info)
         	if (debug >= DBGDETAIL) 
 		{
 			cl_log(LOG_INFO, "error:[%s]", strerror(errno));
-			cl_log(LOG_INFO, "Unable to setuid for recovery of %s\n", info->appname);
+			cl_log(LOG_INFO, "Unable to setuid for recovery of %s", info->appname);
 		}
    	}
 }
